unit ArchonGfx;

interface

uses VAS, StartUp, VGA, Crt, Dos, Keyboard, CrashTest;

type SpriteType = array[0..124] of record
       x, y      : Word;
       RestAnim  : Byte;
       WalkAnim  : array[1..4] of Byte;
       ShotAnim,
       Speed,
       ShotSpeed,
       FirePower,
       Timer,
       Range     : Byte;
       Fly       : Boolean;
     end;

     WizardSpells = array[1..2] of record
       Teleport,
       Heal,
       ShiftTime,
       Exchange,
       Revive,
       Imprison  : Byte;
     end;

const White          =   0;
      Cyan           =   1;
      Green          =   2;
      Magenta        =   3;
      Blue           =   4;
      Black          =   5;
      Magic          =   6;
      Neut           =   7;
      Wizard         =   0;
      Golem          =   7;
      Phoenix        =  14;
      Knight         =  18;
      Unicorn        =  34;
      Djinni         =  39;
      Archer         =  46;
      Valkyrie       =  53;
      Manticore      =  60;
      Goblin         =  67;
      Basilisk       =  82;
      Troll          =  89;
      Siren          =  96;
      Dragon         = 104;
      DarkWizard     = 111;
      FireCritter    = 118;
      Tornado        = 124;
      ScreenSet      : array[0..8, 0..8] of Byte =
        ((Black, White, Black, Neut,  Neut, Neut,  White, Black, White),
         (White, Black, Neut,  White, Neut, Black, Neut,  White, Black),
         (Black, Neut,  White, Black, Neut, White, Black, Neut,  White),
         (Neut,  White, Black, White, Neut, Black, White, Black, Neut ),
         (White, Neut,  Neut,  Neut,  Neut, Neut,  Neut,  Neut,  Black),
         (Neut,  White, Black, White, Neut, Black, White, Black, Neut ),
         (Black, Neut,  White, Black, Neut, White, Black, Neut,  White),
         (White, Black, Neut,  White, Neut, Black, Neut,  White, Black),
         (Black, White, Black, Neut,  Neut, Neut,  White, Black, White));

      ItemSet        : array[0..8, 0..8] of Byte =
        ((Valkyrie, Archer, 255, 255, 255, 255, 255, Manticore, Siren),
         (Golem,    Knight, 255, 255, 255, 255, 255, Goblin,    Troll),
         (Unicorn,  Knight, 255, 255, 255, 255, 255, Goblin,    Basilisk),
         (Djinni,   Knight, 255, 255, 255, 255, 255, Goblin,    FireCritter),
         (Wizard,   Knight, 255, 255, 255, 255, 255, Goblin,    DarkWizard),
         (Phoenix,  Knight, 255, 255, 255, 255, 255, Goblin,    Dragon),
         (Unicorn,  Knight, 255, 255, 255, 255, 255, Goblin,    Basilisk),
         (Golem,    Knight, 255, 255, 255, 255, 255, Goblin,    Troll),
         (Valkyrie, Archer, 255, 255, 255, 255, 255, Manticore, Siren));

var SprInfo      : SpriteType;
    E, S,
    GameOver     : VASPtr;
    P            : array[1..2] of VASPtr;
    OnePlayer,
    CompDark,
    LightFirst,
    FirstRestore,
    DoExit,
    CheckExit    : Boolean;
    Counter      : Byte;
    x, y,
    Winner       : Word;
    Quit         : Boolean;
    StartDir     : String;

procedure Initialize(ReInit : Boolean);
procedure Done;
procedure LoadFont;
procedure TermFont;
procedure LoadSprites;
procedure GfxWrite(St : String; X, Y : Word);
procedure WriteMsg(St : String);
procedure FlipXRange(s, e : Byte; V : VASPtr);
procedure FlipYRange(s, e : Byte; V : VASPtr);
procedure PutRestAnim;
procedure Walk(SprNum, Dir, Count : Byte; ResetSprite, WalkRestore : Boolean);
procedure WalkUp;
procedure MainMenu;
procedure ShowCursor;
procedure HideCursor;
procedure InitFight(var Art, x, y, Winner : Word);
procedure MoveCursor(d : Byte);
function  IsSprite : Boolean;
procedure DrawNeutralColor;
procedure StartGame;

implementation

const SpellData : array[1..7] of String = ('Teleport an item.',
                                           'Heal an item.',
                                           'Shift time.',
                                           'Exchange items.',
                                           'Revive an item.',
                                           'Imprison an item.',
                                           'Do not cast a spell.');

var FP           : VasPtr;
    Bg           : array[1..2] of BkgrPtr;
    CursorX,
    CursorY,
    CursorOldX,
    CursorOldY   : Word;
    OldInt8      : procedure;
    Walked,
    Pl1Turn,
    PlChange     : Boolean;
    PtrNumber,
    RestAnim,
    NeutralColor : Byte;
    OldX, OldY   : Word;
    ArchonPal    : Pointer;
    ShotTimer    : array[1..2] of Byte;
    KilledItems  : array[1..2, 1..8] of Byte;
    Spell        : WizardSpells;
    Imprisoned   : array[0..8, 0..8] of Boolean;

{$F+}
procedure NewInt8; interrupt;
var i : Byte;
begin
  if CheckExit then if Port[$60] < $80 then DoExit := True;
  if Counter < 255 then Inc(Counter);
  for i := 1 to 2 do if ShotTimer[i] < 90 then Inc(ShotTimer[i]);
  Inline($9C);
  OldInt8;
end;
{$F-}

procedure RMove(SX, SY, W, H, DX, DY : Word);
begin
  Dec(SX, SX mod 4);
  Inc(W, W mod 4);
  Dec(DX, DX mod 4);
  SX := SX div 4;
  W := W div 4;
  DX := DX  div 4;
  XMove(SX, SY, DX, DY, W, H, 0, 0, 0);
end;

procedure SetOverscan(c : Byte); assembler;
asm
   mov      ax, 1001h
   mov      bh, c
   int      10h
end;

procedure Initialize(ReInit : Boolean);
var i, a, b : Byte;
begin
  GetDir(0, StartDir);
  if not ReInit then begin
    DisableWarmBoot;
    SetCBreak(False);
    LoadingMsg('Loading "ARCHON GOLD"... Copyright (c) 1992 by MEGA HACK');
    LoadFont;
    LoadSprites;
    GetMem(ArchonPal, 768);
    LoadMem(StartDir+'\GRAPHX\ARCHON.PAL', ArchonPal, 0, 768);
    for i := 1 to 2 do New(Bg[i], InitBkgr(7, 20));
    InitNGraph(N320x200, 80);
  end;
  DoExit := False;
  OnePlayer := False;
  CompDark := True;
  LightFirst := True;
  CursorX := 34; CursorY := 82;
  CheckExit := False;
  Counter := 0;
  Randomize;
  NeutralColor := Random(6);
  FillChar(KilledItems, SizeOf(KilledItems), 255);
  FillChar(Imprisoned, SizeOf(Imprisoned), 0);
  for i := 1 to 2 do begin
    Spell[i].Teleport  := 2;
    Spell[i].Heal      := 1;
    Spell[i].ShiftTime := 2;
    Spell[i].Exchange  := 2;
    Spell[i].Revive    := 1;
    Spell[i].Imprison  := 1;
  end;
  with SprInfo[Wizard] do begin
    RestAnim := 0;
    WalkAnim[1] := 2;
    WalkAnim[2] := 4;
    WalkAnim[3] := 0;
    WalkAnim[4] := 0;
    Speed := 2;
    ShotSpeed := 16;
    ShotAnim := 16;
    FirePower := 117;
    Timer := 18;
    Fly := True;
    Range := 3;
  end;
  with SprInfo[Golem] do begin
    RestAnim := 7;
    WalkAnim[1] :=  9;
    WalkAnim[2] := 11;
    WalkAnim[3] :=  7;
    WalkAnim[4] :=  7;
    Speed := 1;
    ShotSpeed := 10;
    ShotAnim := 16;
    FirePower := 117;
    Timer := 36;
    Fly := False;
    Range := 3;
  end;
  with SprInfo[Phoenix] do begin
    RestAnim := 14;
    WalkAnim[1] := 16;
    WalkAnim[2] := 16;
    WalkAnim[3] := 14;
    WalkAnim[4] := 14;
    Speed := 2;
    ShotSpeed := 14;
    ShotAnim := 12;
    FirePower := 88;
    Timer := 27;
    Fly := True;
    Range := 5;
  end;
  with SprInfo[Knight] do begin
    RestAnim := 18;
    WalkAnim[1] := 20;
    WalkAnim[2] := 22;
    WalkAnim[3] := 18;
    WalkAnim[4] := 18;
    Speed := 2;
    ShotSpeed := 0;
    ShotAnim := 0;
    FirePower := 70;
    Timer := 15;
    Fly := False;
    Range := 3;
  end;
  with SprInfo[Unicorn] do begin
    RestAnim := 34;
    WalkAnim[1] := 36;
    WalkAnim[2] := 36;
    WalkAnim[3] := 34;
    WalkAnim[4] := 34;
    Speed := 2;
    ShotSpeed := 16;
    ShotAnim := 12;
    FirePower := 88;
    Timer := 27;
    Fly := False;
    Range := 4;
  end;
  with SprInfo[Djinni] do begin
    RestAnim := 39;
    WalkAnim[1] := 41;
    WalkAnim[2] := 43;
    WalkAnim[3] := 39;
    WalkAnim[4] := 39;
    Speed := 2;
    ShotSpeed := 13;
    ShotAnim := 20;
    FirePower := 88;
    Timer := 27;
    Fly := True;
    Range :=  4
  end;
  with SprInfo[Archer] do begin
    RestAnim := 46;
    WalkAnim[1] := 48;
    WalkAnim[2] := 50;
    WalkAnim[3] := 46;
    WalkAnim[4] := 46;
    Speed := 2;
    ShotSpeed := 12;
    ShotAnim := 8;
    FirePower := 88;
    Timer := 30;
    Fly := False;
    Range := 3;
  end;
  with SprInfo[Valkyrie] do begin
    RestAnim := 53;
    WalkAnim[1] := 55;
    WalkAnim[2] := 57;
    WalkAnim[3] := 53;
    WalkAnim[4] := 53;
    Speed := 1;
    ShotSpeed := 11;
    ShotAnim := 4;
    FirePower := 88;
    Timer := 35;
    Fly := True;
    Range := 3;
  end;
  with SprInfo[Manticore] do begin
    RestAnim := 60;
    WalkAnim[1] := 62;
    WalkAnim[2] := 64;
    WalkAnim[3] := 60;
    WalkAnim[4] := 60;
    Speed := 2;
    ShotSpeed := 12;
    ShotAnim := 0;
    FirePower := 88;
    Timer := 30;
    Fly := False;
    Range := 3;
  end;
  with SprInfo[Goblin] do begin
    RestAnim := 67;
    WalkAnim[1] := 69;
    WalkAnim[2] := 71;
    WalkAnim[3] := 67;
    WalkAnim[4] := 67;
    Speed := 2;
    ShotSpeed := 0;
    ShotAnim := 0;
    FirePower := 70;
    Timer := 15;
    Fly := False;
    Range := 3;
  end;
  with SprInfo[Basilisk] do begin
    RestAnim := 82;
    WalkAnim[1] := 84;
    WalkAnim[2] := 86;
    WalkAnim[3] := 82;
    WalkAnim[4] := 82;
    Speed := 2;
    ShotSpeed := 16;
    ShotAnim := 4;
    FirePower := 88;
    Timer := 27;
    Fly := False;
    Range := 4;
  end;
  with SprInfo[Troll] do begin
    RestAnim := 89;
    WalkAnim[1] := 91;
    WalkAnim[2] := 93;
    WalkAnim[3] := 89;
    WalkAnim[4] := 89;
    Speed := 1;
    ShotSpeed := 10;
    ShotAnim := 16;
    FirePower := 117;
    Timer := 36;
    Fly := False;
    Range := 3;
  end;
  with SprInfo[Siren] do begin
    RestAnim := 96;
    WalkAnim[1] :=  98;
    WalkAnim[2] := 100;
    WalkAnim[3] :=  96;
    WalkAnim[4] :=  96;
    Speed := 2;
    ShotSpeed := 0;
    ShotAnim := 0;
    FirePower := 59;
    Timer := 18;
    Fly := True;
    Range := 3;
  end;
  with SprInfo[FireCritter] do begin
    RestAnim := 118;
    WalkAnim[1] := 120;
    WalkAnim[2] := 122;
    WalkAnim[3] := 118;
    WalkAnim[4] := 118;
    Speed := 2;
    ShotSpeed := 14;
    ShotAnim := 4;
    FirePower := 88;
    Timer := 27;
    Fly := True;
    Range := 4;
  end;
  with SprInfo[Dragon] do begin
    RestAnim := 104;
    WalkAnim[1] := 106;
    WalkAnim[2] := 108;
    WalkAnim[3] := 104;
    WalkAnim[4] := 104;
    Speed := 2;
    ShotSpeed := 13;
    ShotAnim := 4;
    FirePower := 175;
    Timer := 27;
    Fly := True;
    Range := 5;
  end;
  with SprInfo[DarkWizard] do begin
    RestAnim := 111;
    WalkAnim[1] := 113;
    WalkAnim[2] := 115;
    WalkAnim[3] := 111;
    WalkAnim[4] := 111;
    Speed := 2;
    ShotSpeed := 16;
    ShotAnim := 16;
    FirePower := 117;
    Timer := 18;
    Fly := True;
    Range := 3;
  end;
end;

procedure Done;
var i : Byte;
begin
  SetIntVec($08, @OldInt8);
  TermFont;
  for i := 1 to 2 do begin
    P[i]^.DisposeAllSprites;
    Dispose(Bg[i], DisposeBkgr);
  end;
  E^.DisposeAllSprites;
  S^.DisposeAllSprites;
  GameOver^.DisposeAllSprites;
  Dispose(ArchonPal);
  asm
     mov      ax, 03h
     int      10h
  end;
  WriteLn('Exiting "ARCHON GOLD"...Copyright (c) 1992 by MEGA HACK');
  WriteLn;
  Halt;
end;

procedure LoadFont;
begin
  New(FP, InitVAS);
  FP^.LoadSprites(StartDir+'\GRAPHX\GREYFONT.FNT', 0, 70);
end;

procedure TermFont;
begin
  FP^.DisposeAllSprites;
end;

procedure LoadSprites;
var i : Byte;
begin
  for i := 1 to 2 do New(P[i], InitVas);
  New(E, InitVas);
  New(S, InitVas);
  New(GameOver, InitVas);
  P[1]^.LoadSprites(StartDir+'\SPRITES\ARCHSPR.SPR', 0, 96);
  P[2]^.LoadSprites(StartDir+'\SPRITES\ARCHSPR2.SPR', 0, 30);
  E^.LoadSprites(StartDir+'\SPRITES\ARCHEXT.SPR', 0, 7);
  S^.LoadSprites(StartDir+'\SPRITES\BULLETS.SPR', 0, 24);
  GameOver^.LoadSprites(StartDir+'\SPRITES\GAMEOVER.SPR', 0, 2);
end;

procedure GfxWrite(St : String; X, Y : Word);
var i : Byte;
begin
  for i := 0 to Length(St)-1 do begin
    case St[i+1] of
      'A'..'Z' : FP^.PutSprite(X+i*8, Y, Ord(St[i+1])-65, 1);
      'a'..'z' : FP^.PutSprite(X+i*8, Y, Ord(St[i+1])-71, 1);
      '1'..'9' : FP^.PutSprite(X+i*8, Y, Ord(St[i+1])+3, 1);
      '0'      : FP^.PutSprite(X+i*8, Y, 61, 1);
      '.'      : FP^.PutSprite(X+i*8, Y, 62, 1);
      ','      : FP^.PutSprite(X+i*8, Y, 63, 1);
      ':'      : FP^.PutSprite(X+i*8, Y, 64, 1);
      '-'      : FP^.PutSprite(X+i*8, Y, 65, 1);
      '*'      : FP^.PutSprite(X+i*8, Y, 66, 1);
      '!'      : FP^.PutSprite(X+i*8, Y, 67, 1);
      '?'      : FP^.PutSprite(X+i*8, Y, 68, 1);
    end;
    Inc(X);
  end;
end;

procedure ClearMsg;
begin
  WaitVRetrace;
  Stripe(38,     187, 244, 9, 255);
  Stripe(38, 200+187, 244, 9, 255);
end;

procedure WriteMsg(St : String);
begin
  WaitVRetrace;
  Stripe(38,     187, 244, 9, 255);
  Stripe(38, 200+187, 244, 9, 255);
  GfxWrite(St, 160-(Length(St)*8+Length(St)) div 2,     188);
  GfxWrite(St, 160-(Length(St)*8+Length(St)) div 2, 200+188);
end;

procedure FlipXRange(s, e : Byte; V : VasPtr);
var i : Byte;
begin
  for i := s to e do V^.FlipSpriteX(i);
end;

procedure FlipYRange(s, e : Byte; V : VasPtr);
var i : Byte;
begin
  for i := s to e do V^.FlipSpriteY(i);
end;

procedure Walk(SprNum, Dir, Count : Byte; ResetSprite, WalkRestore : Boolean);
var Anim,
    Tmp, c,
    Moved      : Byte;

begin
  if DoExit then Exit;
  Walked := True;
  x := SprInfo[SprNum].x; y := SprInfo[SprNum].y;
  OldX := x; OldY := y;
  Anim := SprInfo[SprNum].WalkAnim[Dir];
  Tmp := Anim;
  RestAnim := SprInfo[SprNum].RestAnim;
  c := 0;
  Moved := 0;
  
  if WalkRestore then begin
    Bg[1]^.SaveBkgr(x, 400+y);
    Bg[2]^.SaveBkgr(x, 400+y);
  end;
  
  if SprNum >= 96 then begin
    SprNum := SprNum - 96;
    PtrNumber := 2;
    Anim := Anim - 96;
    Tmp := Anim;
    RestAnim := RestAnim - 96;
  end else PtrNumber := 1;
  
  if Dir = 3 then FlipXRange(SprNum, SprNum+1, P[PtrNumber]);
  if (SprNum = Phoenix) and (Dir = 2) then FlipYRange(16, 17, P[1]);
  if (SprNum = Unicorn) and (Dir = 2) then FlipYRange(36, 37, P[1]);
  
  repeat
    Bg[2]^.RestoreBkgr(OldX, 200+OldY);
    Bg[2]^.SaveBkgr(x, 200+y);
    P[PtrNumber]^.PutSprite(x, 200+y, Anim, 1);
    SetScrPos(0, 200);
    Bg[1]^.RestoreBkgr(OldX, OldY);
    Bg[1]^.SaveBkgr(x, y);
    P[PtrNumber]^.PutSprite(x, y, Anim, 1);
    SetScrPos(0, 0);
    OldX := x;
    OldY := y;
    case Dir of
      1 : Dec(y, 2);
      2 : Inc(y, 2);
      3 : Dec(x, 2);
      4 : Inc(x, 2);
    end;
    Inc(Moved);
    if Moved mod 4 = 0 then if Anim = Tmp then Inc(Anim)
                              else Anim := Tmp;
    if (Moved = 11) and ((Dir = 1) or (Dir = 2)) then begin
      Inc(c);
      Moved := 1;
    end;
    if (Moved = 15) and ((Dir = 3) or (Dir = 4)) then begin
      Inc(c);
      Moved := 1;
    end;
  until c = Count;

  if Dir = 3 then FlipXRange(SprNum, SprNum+1, P[PtrNumber]);
  if PtrNumber = 2 then SprNum := SprNum + 96;
  if (SprNum = Phoenix) and (Dir = 2) then FlipYRange(16, 17, P[1]);
  if (SprNum = Unicorn) and (Dir = 2) then FlipYRange(36, 37, P[1]);

  SprInfo[SprNum].x := OldX; SprInfo[SprNum].y := OldY;

  if ResetSprite then PutRestAnim;

  Counter := 0;
  FirstRestore := False;
end;

procedure SetUpSprites;
var i : Word;
procedure FlipSprites;
begin
  P[2]^.FlipSpriteX(Siren-96);
  P[1]^.FlipSpriteX(Troll);
  P[1]^.FlipSpriteX(Basilisk);
  P[2]^.FlipSpriteX(FireCritter-96);
  P[2]^.FlipSpriteX(DarkWizard-96);
  P[2]^.FlipSpriteX(Dragon-96);
  P[1]^.FlipSpriteX(Manticore);
  P[1]^.FlipSpriteX(Goblin);
end;

begin
  P[1]^.PutSprite(34,   2, Valkyrie, 1);
  P[1]^.PutSprite(34,  22, Golem, 1);
  P[1]^.PutSprite(34,  42, Unicorn, 1);
  P[1]^.PutSprite(34,  62, Djinni, 1);
  P[1]^.PutSprite(34,  82, Wizard, 1);
  P[1]^.PutSprite(34, 102, Phoenix, 1);
  P[1]^.PutSprite(34, 122, Unicorn, 1);
  P[1]^.PutSprite(34, 142, Golem, 1);
  P[1]^.PutSprite(34, 162, Valkyrie, 1);
  P[1]^.PutSprite(62,   2, Archer, 1);
  for i := 1 to 7 do P[1]^.PutSprite(62, 2+i*20, Knight, 1);
  P[1]^.PutSprite(62, 162, Archer, 1);
  FlipSprites;
  P[2]^.PutSprite(258,   2, Siren-96, 1);
  P[1]^.PutSprite(258,  22, Troll, 1);
  P[1]^.PutSprite(258,  42, Basilisk, 1);
  P[2]^.PutSprite(258,  62, FireCritter-96, 1);
  P[2]^.PutSprite(258,  82, DarkWizard-96, 1);
  P[2]^.PutSprite(258, 102, Dragon-96, 1);
  P[1]^.PutSprite(258, 122, Basilisk, 1);
  P[1]^.PutSprite(258, 142, Troll, 1);
  P[2]^.PutSprite(258, 162, Siren-96, 1);
  P[1]^.PutSprite(230,   2, Manticore, 1);
  for i := 1 to 7 do P[1]^.PutSprite(230, 2+i*20, Goblin, 1);
  P[1]^.PutSprite(230, 162, Manticore, 1);
  RMove(CursorOldX, CursorOldY, 32, 24, 0, 600);
  FlipSprites;
end;

procedure WalkUp;
var i : Byte;

begin
  ShowPCX(StartDir+'\GRAPHX\ARCHON.GFX', 0, 0, 1);
  RMove(0, 0, 320, 200, 0, 200);
  RMove(0, 0, 320, 200, 0, 400);
  Blend(True);
  CheckExit := True;
  WriteMsg('Knight');
  for i := 1 to 7 do begin
    SprInfo[Knight].x := 7; SprInfo[Knight].y := 82;
    Walk(Knight, 4, 1, True, True);
    if i <= 3 then Walk(Knight, 1, 4-i, True, True)
      else if i > 4 then Walk(Knight, 2, -4+i, True, True);
    Walk(Knight, 4, 1, True, True);
  end;

  WriteMsg('Archer');
  for i := 1 to 2 do begin
    SprInfo[Archer].x := 7; SprInfo[Archer].y := 82;
    Walk(Archer, 4, 1, True, True);
    if i = 1 then Walk(Archer, 1, 4, True, True)
      else Walk(Archer, 2, 4, True, True);
    Walk(Archer, 4, 1, True, True);
  end;

  WriteMsg('Valkyrie');
  for i := 1 to 2 do begin
    SprInfo[Valkyrie].x := 7; SprInfo[Valkyrie].y := 82;
    Walk(Valkyrie, 4, 1, True, True);
    if i = 1 then Walk(Valkyrie, 1, 4, True, True)
      else Walk(Valkyrie, 2, 4, True, True);
  end;

  WriteMsg('Golem');
  for i := 1 to 2 do begin
    SprInfo[Golem].x := 7; SprInfo[Golem].y := 82;
    Walk(Golem, 4, 1, True, True);
    if i = 1 then Walk(Golem, 1, 3, True, True)
      else Walk(Golem, 2, 3, True, True);
  end;

  WriteMsg('Unicorn');
  for i := 1 to 2 do begin
    SprInfo[Unicorn].x := 7; SprInfo[Unicorn].y := 82;
    Walk(Unicorn, 4, 1, True, True);
    if i = 1 then Walk(Unicorn, 1, 2, True, True)
      else Walk(Unicorn, 2, 2, True, True);
  end;

  WriteMsg('Djinni');
  SprInfo[Djinni].x := 7; SprInfo[Djinni].y := 82;
  Walk(Djinni, 4, 1, True, True);
  Walk(Djinni, 1, 1, True, True);

  WriteMsg('Phoenix');
  SprInfo[Phoenix].x := 7; SprInfo[Phoenix].y := 82;
  Walk(Phoenix, 4, 1, True, True);
  Walk(Phoenix, 2, 1, True, True);

  WriteMsg('Wizard');
  SprInfo[Wizard].x := 7; SprInfo[Wizard].y := 82;
  Walk(Wizard, 4, 1, True, True);

  WriteMsg('Goblin');
  for i := 1 to 7 do begin
    SprInfo[Goblin].x := 286; SprInfo[Goblin].y := 82;
    Walk(Goblin, 3, 1, True, True);
    if i <= 3 then Walk(Goblin, 1, 4-i, True, True)
      else if i > 4 then Walk(Goblin, 2, -4+i, True, True);
    Walk(Goblin, 3, 1, True, True);
  end;

  WriteMsg('Manticore');
  for i := 1 to 2 do begin
    SprInfo[Manticore].x := 286; SprInfo[Manticore].y := 82;
    Walk(Manticore, 3, 1, True, True);
    if i = 1 then Walk(Manticore, 1, 4, True, True)
      else Walk(Manticore, 2, 4, True, True);
    Walk(Manticore, 3, 1, True, True);
  end;

  WriteMsg('Siren');
  for i := 1 to 2 do begin
    SprInfo[Siren].x := 286; SprInfo[Siren].y := 82;
    Walk(Siren, 3, 1, True, True);
    if i = 1 then Walk(Siren, 1, 4, True, True)
      else Walk(Siren, 2, 4, True, True);
  end;

  WriteMsg('Troll');
  for i := 1 to 2 do begin
    SprInfo[Troll].x := 286; SprInfo[Troll].y := 82;
    Walk(Troll, 3, 1, True, True);
    if i = 1 then Walk(Troll, 1, 3, True, True)
      else Walk(Troll, 2, 3, True, True);
  end;

  WriteMsg('Basilisk');
  for i := 1 to 2 do begin
    SprInfo[Basilisk].x := 286; SprInfo[Basilisk].y := 82;
    Walk(Basilisk, 3, 1, True, True);
    if i = 1 then Walk(Basilisk, 1, 2, True, True)
      else Walk(Basilisk, 2, 2, True, True);
  end;

  WriteMsg('Fire Critter');
  SprInfo[FireCritter].x := 286; SprInfo[FireCritter].y := 82;
  Walk(FireCritter, 3, 1, True, True);
  Walk(FireCritter, 1, 1, True, True);

  WriteMsg('Dragon');
  SprInfo[Dragon].x := 286; SprInfo[Dragon].y := 82;
  Walk(Dragon, 3, 1, True, True);
  Walk(Dragon, 2, 1, True, True);

  WriteMsg('Wizard');
  SprInfo[DarkWizard].x := 286; SprInfo[DarkWizard].y := 82;
  Walk(DarkWizard, 3, 1, True, True);

  DoExit := False;
  CheckExit := False;

  Screen(0);
  RMove(0, 400, 320, 200, 0, 0);
  SetUpSprites;
  RMove(0, 0, 320, 200, 0, 200);
  Screen(1);
end;

procedure MainMenu;
var Ch   : Char;
    i, j : Word;

begin
  Blend(False);
  ShowPCX(StartDir+'\GRAPHX\ARCHON.GFX', 0, 0, 2);
  SetOverScan(0);
  Blend(True);
  repeat
    Ch := ReadKey;
  until Ch in [#59..#61];
  case Ch of
    #59 : LightFirst := True;
    #60 : LightFirst := False;
    #61 : begin Blend(False); Done; end;
  end;
  if LightFirst then begin
    CursorX := 34;
    Pl1Turn := True;
  end else begin
        CursorX := 258;
        Pl1Turn := False;
      end;
  CursorY := 82;
end;

procedure ShowCursor;
begin
  RMove(CursorX-2, CursorY-2, 32, 24, 0, 600);
  Rect(CursorX-2, CursorY-2, 32, 24, 35);
  Rect(CursorX-1, CursorY-1, 30, 22, 33);
end;

procedure HideCursor;
begin
  RMove(0, 600, 32, 24, CursorX-2, CursorY-2);
end;

procedure MoveCursor(d : Byte);
begin
  if ((d = 1) and (CursorY =   2)) or
     ((d = 2) and (CursorY = 162)) or
     ((d = 3) and (CursorX =  34)) or
     ((d = 4) and (CursorX = 258)) then Exit;
  RMove(0, 600, 32, 24, CursorX-2, CursorY-2);
  case d of
    1 : if CursorY >   2 then Dec(CursorY, 20);
    2 : if CursorY < 162 then Inc(CursorY, 20);
    3 : if CursorX >  34 then Dec(CursorX, 28);
    4 : if CursorX < 258 then Inc(CursorX, 28);
  end;
  RMove(CursorX-2, CursorY-2, 32, 24, 0, 600);
  Rect(CursorX-2, CursorY-2, 32, 24, 35);
  Rect(CursorX-1, CursorY-1, 30, 22, 33);
end;

function IsSprite : Boolean;
begin
  IsSprite := True;
  if ItemSet[(CursorY-2) div 20, (CursorX-34) div 28] = 255 then IsSprite := False;
end;

procedure Fight(Art1, Art2 : Word; var Winner : Word);
var B                  : BkgrPtr;
    Backgr,
    ShotBkgr,
    SwordBkgr          : array[1..4] of BkgrPtr;
    i, j, PtrNumber    : Byte;
    LastAnim,
    RestAnim,
    Moved, Speed,
    Art, Dir,
    ShotDir,
    ShotSpeed,
    LastDir            : array[1..2] of Byte;
    x, y,
    OldX, OldY,
    ShotX, ShotY,
    ShotOldX, ShotOldY : array[1..2] of Word;
    Flipped,
    FlippedUD,
    Shooting           : array[1..2] of Boolean;
    ChangeCheck        : Boolean;
    Energy             : array[1..2] of Word;

procedure SetUpFighters(Light, Dark : Word);
var i : Byte;
begin
  Art[1] := Light; Art[2] := Dark;
  x[1] :=  10; y[1] := 161;
  x[2] := 322; y[2] := 161;
  OldX[1] := x[1]; OldY[1] := y[1];
  OldX[2] := x[2]; OldY[2] := y[2];
  LastAnim[1] := 0; LastAnim[2] := 0;
  Moved[1] := 0; Moved[2] := 0;
  Speed[1] := SprInfo[Light].Speed; Speed[2] := SprInfo[Dark].Speed;
  ShotSpeed[1] := SprInfo[Light].ShotSpeed; ShotSpeed[2] := SprInfo[Dark].ShotSpeed;
  Dir[1] := 4; Dir[2] := 3;

  RestAnim[1] := SprInfo[Light].RestAnim;
  RestAnim[2] := SprInfo[Dark].RestAnim;

  if Art[2] >= 96 then begin
    PtrNumber := 2;
    RestAnim[2] := RestAnim[2]-96;
    for i := 1 to 4 do
      SprInfo[Art[2]].WalkAnim[i] := SprInfo[Art[2]].WalkAnim[i]-96;
  end else PtrNumber := 1;

  for i := 1 to 4 do begin
    New(Backgr[i], InitBkgr(7, 20));
    New(ShotBkgr[i], InitBkgr(3, 7));
    New(SwordBkgr[i], InitBkgr(7, 20));
  end;
  BackGr[1]^.SaveBkgr( 10, 161); BackGr[2]^.SaveBkgr( 10, 350+161);
  BackGr[3]^.SaveBkgr(322, 161); BackGr[4]^.SaveBkgr(322, 350+161);
  P[1]^.PutSprite(10,     161, RestAnim[1], 1);
  P[1]^.PutSprite(10, 350+161, RestAnim[1], 1);
  P[PtrNumber]^.FlipSpriteX(RestAnim[2]);
  P[PtrNumber]^.PutSprite(322,     161, RestAnim[2], 1);
  P[PtrNumber]^.PutSprite(322, 350+161, RestAnim[2], 1);
  P[PtrNumber]^.FlipSpriteX(RestAnim[2]);
  Flipped[1] := False; Flipped[2] := False;
  FlippedUD[1] := False; FlippedUD[2] := False;
  Shooting[1] := False; Shooting[2] := False;
  Energy[1] := 350; Energy[2] := 350;
  LastDir[1] := 4; LastDir[2] := 3;
end;

function Changed(i : Byte) : Boolean;
begin
  Changed := False;
  if (OldX[i] <> x[i]) or (OldY[i] <> y[i]) then Changed := True;
end;

function MayGo(n, Dir : Byte) : Boolean;
var i, j, Speed : Byte;
    x, y, w, h  : Word;
begin
  MayGo := True;
  case Dir of
    1 : if (GetPixel(OldX[n]+05, OldY[n]-02) < 127) or
           (GetPixel(OldX[n]+14, OldY[n]-02) < 127) or
           (GetPixel(OldX[n]+20, OldY[n]-02) < 127) then MayGo := False;
    2 : if (GetPixel(OldX[n]+05, OldY[n]+21) < 127) or
           (GetPixel(OldX[n]+14, OldY[n]+21) < 127) or
           (GetPixel(OldX[n]+20, OldY[n]+21) < 127) then MayGo := False;
    3 : if (GetPixel(OldX[n]+00, OldY[n]+02) < 127) or
           (GetPixel(OldX[n]+00, OldY[n]+10) < 127) or
           (GetPixel(OldX[n]+00, OldY[n]+18) < 127) then MayGo := False;
    4 : if (GetPixel(OldX[n]+28, OldY[n]+02) < 127) or
           (GetPixel(OldX[n]+28, OldY[n]+10) < 127) or
           (GetPixel(OldX[n]+28, OldY[n]+18) < 127) then MayGo := False;
    5 : if (GetPixel(OldX[n]+00, OldY[n]+02) < 127) or
           (GetPixel(OldX[n]+00, OldY[n]+10) < 127) or
           (GetPixel(OldX[n]+00, OldY[n]+18) < 127) or
           (GetPixel(OldX[n]+05, OldY[n]-02) < 127) or
           (GetPixel(OldX[n]+14, OldY[n]-02) < 127) or
           (GetPixel(OldX[n]+20, OldY[n]-02) < 127) then MayGo := False;
    6 : if (GetPixel(OldX[n]+28, OldY[n]+02) < 127) or
           (GetPixel(OldX[n]+28, OldY[n]+10) < 127) or
           (GetPixel(OldX[n]+28, OldY[n]+18) < 127) or
           (GetPixel(OldX[n]+05, OldY[n]-02) < 127) or
           (GetPixel(OldX[n]+14, OldY[n]-02) < 127) or
           (GetPixel(OldX[n]+20, OldY[n]-02) < 127) then MayGo := False;
    7 : if (GetPixel(OldX[n]+00, OldY[n]+02) < 127) or
           (GetPixel(OldX[n]+00, OldY[n]+10) < 127) or
           (GetPixel(OldX[n]+00, OldY[n]+18) < 127) or
           (GetPixel(OldX[n]+05, OldY[n]+21) < 127) or
           (GetPixel(OldX[n]+14, OldY[n]+21) < 127) or
           (GetPixel(OldX[n]+20, OldY[n]+21) < 127) then MayGo := False;
    8 : if (GetPixel(OldX[n]+28, OldY[n]+02) < 127) or
           (GetPixel(OldX[n]+28, OldY[n]+10) < 127) or
           (GetPixel(OldX[n]+28, OldY[n]+18) < 127) or
           (GetPixel(OldX[n]+05, OldY[n]+21) < 127) or
           (GetPixel(OldX[n]+14, OldY[n]+21) < 127) or
           (GetPixel(OldX[n]+20, OldY[n]+21) < 127) then MayGo := False;
  end;
end;

procedure SetNewPos(PlNum, d : Byte);
begin
  case PlNum of
    1 : case d of
          1 : begin
                Dec(y[1], Speed[1]); Inc(Moved[1]); Dir[1] := 1;
                if (Art[1] = Phoenix) or (Art[1] = Unicorn) then
                  if FlippedUD[1] then begin
                    FlipYRange(Art[1]+2, Art[1]+3, P[1]);
                    FlippedUD[1] := False;
                  end;
              end;
          2 : begin
                Inc(y[1], Speed[1]); Inc(Moved[1]); Dir[1] := 2;
                if (Art[1] = Phoenix) or (Art[1] = Unicorn) then
                  if not FlippedUD[1] then begin
                    FlipYRange(Art[1]+2, Art[1]+3, P[1]);
                    FlippedUD[1] := True;
                  end;
              end;
          3 : begin
                Dec(x[1], Speed[1]); Inc(Moved[1]); Dir[1] := 3;
                if not Flipped[1] then begin
                  FlipXRange(Art[1], Art[1]+1, P[1]);
                  Flipped[1] := True;
                end;
              end;
          4 : begin
                Inc(x[1], Speed[1]); Inc(Moved[1]); Dir[1] := 4;
                if Flipped[1] then begin
                  FlipXRange(Art[1], Art[1]+1, P[1]);
                  Flipped[1] := False;
                end;
              end;
          5 : begin
                Dec(x[1], Speed[1]); Dec(y[1], Speed[1]); Inc(Moved[1]);
                Dir[1] := 3;
                if not Flipped[1] then begin
                  FlipXRange(Art[1], Art[1]+1, P[1]);
                  Flipped[1] := True;
                end;
              end;
          6 : begin
                Inc(x[1], Speed[1]); Dec(y[1], Speed[1]); Inc(Moved[1]);
                Dir[1] := 4;
                if Flipped[1] then begin
                  FlipXRange(Art[1], Art[1]+1, P[1]);
                  Flipped[1] := False;
                end;
              end;
          7 : begin
                Dec(x[1], Speed[1]); Inc(y[1], Speed[1]); Inc(Moved[1]);
                Dir[1] := 3;
                if not Flipped[1] then begin
                  FlipXRange(Art[1], Art[1]+1, P[1]);
                  Flipped[1] := True;
                end;
              end;
          8 : begin
                Inc(x[1], Speed[1]); Inc(y[1], Speed[1]); Inc(Moved[1]);
                Dir[1] := 4;
                if Flipped[1] then begin
                  FlipXRange(Art[1], Art[1]+1, P[1]);
                  Flipped[1] := False;
                end;
              end;
        end;
    2 : case d of
          1 : begin
                Dec(y[2], Speed[2]); Inc(Moved[2]); Dir[2] := 1;
              end;
          2 : begin
                Inc(y[2], Speed[2]); Inc(Moved[2]); Dir[2] := 2;
              end;
          3 : begin
                Dec(x[2], Speed[2]); Inc(Moved[2]); Dir[2] := 3;
                if not Flipped[2] then begin
                  if PtrNumber = 1 then FlipXRange(Art[2], Art[2]+1, P[1])
                    else FlipXRange(Art[2]-96, Art[2]-95, P[2]);
                  Flipped[2] := True;
                end;
              end;
          4 : begin
                Inc(x[2], Speed[2]); Inc(Moved[2]); Dir[2] := 4;
                if Flipped[2] then begin
                  if PtrNumber = 1 then FlipXRange(Art[2], Art[2]+1, P[1])
                    else FlipXRange(Art[2]-96, Art[2]-95, P[2]);
                  Flipped[2] := False;
                end;
              end;
          5 : begin
                Dec(x[2], Speed[2]); Dec(y[2], Speed[2]); Inc(Moved[2]);
                Dir[2] := 3;
                if not Flipped[2] then begin
                  if PtrNumber = 1 then FlipXRange(Art[2], Art[2]+1, P[1])
                    else FlipXRange(Art[2]-96, Art[2]-95, P[2]);
                  Flipped[2] := True;
                end;
              end;
          6 : begin
                Inc(x[2], Speed[2]); Dec(y[2], Speed[2]); Inc(Moved[2]);
                Dir[2] := 4;
                if Flipped[2] then begin
                  if PtrNumber = 1 then FlipXRange(Art[2], Art[2]+1, P[1])
                    else FlipXRange(Art[2]-96, Art[2]-95, P[2]);
                  Flipped[2] := False;
                end;
              end;
          7 : begin
                Dec(x[2], Speed[2]); Inc(y[2], Speed[2]); Inc(Moved[2]);
                Dir[2] := 3;
                if not Flipped[2] then begin
                  if PtrNumber = 1 then FlipXRange(Art[2], Art[2]+1, P[1])
                    else FlipXRange(Art[2]-96, Art[2]-95, P[2]);
                  Flipped[2] := True;
                end;
              end;
          8 : begin
                Inc(x[2], Speed[2]); Inc(y[2], Speed[2]); Inc(Moved[2]);
                Dir[2] := 4;
                if Flipped[2] then begin
                  if PtrNumber = 1 then FlipXRange(Art[2], Art[2]+1, P[1])
                    else FlipXRange(Art[2]-96, Art[2]-95, P[2]);
                  Flipped[2] := False;
                end;
              end;
        end;
  end;
  LastDir[PlNum] := Dir[PlNum];
end;

procedure Sword1(d : Byte);
var a, b,
    PtrNum   : Byte;
    SwordHit : Boolean;
begin
  case d of
    1 : begin
          a := 26; b := 27;
          ShotX[1] := x[1]; ShotY[1] := y[1]-20;
        end;
    2 : begin
          a := 28; b := 29;
          ShotX[1] := x[1]; ShotY[1] := y[1]+20;
        end;
    3 : begin
          a := 24; b := 25;
          FlipXRange(24, 25, P[1]);
          ShotX[1] := x[1]-28; ShotY[1] := y[1];
        end;
    4 : begin
          a := 24; b := 25;
          ShotX[1] := x[1]+28; ShotY[1] := y[1];
        end;
    5 : begin
          a := 30; b := 31;
          FlipXRange(30, 31, P[1]);
          ShotX[1] := x[1]-28; ShotY[1] := y[1]-20;
        end;
    6 : begin
          a := 30; b := 31;
          ShotX[1] := x[1]+28; ShotY[1] := y[1]-20;
        end;
    7 : begin
          a := 32; b := 33;
          FlipXRange(32, 33, P[1]);
          ShotX[1] := x[1]-28; ShotY[1] := y[1]+20;
        end;
    8 : begin
          a := 32; b := 33;
          ShotX[1] := x[1]+28; ShotY[1] := y[1]+20;
        end;
  end;
  if Art[2] >= 96 then PtrNum := 2 else PtrNum := 1;
  ShotDir[1] := d;
  ShotOldX[1] := ShotX[1]; ShotOldY[1] := ShotY[1];
  SetScrPos(0, 350);
  BackGr[1]^.RestoreBkgr(OldX[1], OldY[1]);
  BackGr[3]^.RestoreBkgr(OldX[2], OldY[2]);
  SwordBkgr[1]^.SaveBkgr(ShotX[1], ShotY[1]);
  P[PtrNum]^.PutSprite(OldX[2], OldY[2], SprInfo[Art[2]].WalkAnim[LastDir[2]], 1);
  P[1]^.PutSprite(OldX[1], OldY[1], a, 1);
  P[1]^.PutSprite(ShotX[1], ShotY[1], b, 1);
  SetScrPos(0, 0);
  BackGr[2]^.RestoreBkgr(OldX[1], 350+OldY[1]);
  BackGr[4]^.RestoreBkgr(OldX[2], 350+OldY[2]);
  SwordBkgr[2]^.SaveBkgr(ShotX[1], 350+ShotY[1]);
  P[PtrNum]^.PutSprite(OldX[2], 350+OldY[2], SprInfo[Art[2]].WalkAnim[LastDir[2]], 1);
  P[1]^.PutSprite(OldX[1], 350+OldY[1], a, 1);
  P[1]^.PutSprite(ShotX[1], 350+ShotY[1], b, 1);
  SwordHit := False;
  case d of
    1, 2       : SwordHit := TestCrash(ShotX[1]+8, ShotY[1]  , 12, 20, x[2]+8, y[2], 12, 16);
    3, 4       : SwordHit := TestCrash(ShotX[1]  , ShotY[1]+5, 28, 20, x[2]+8, y[2], 12, 16);
    5, 6, 7, 8 : SwordHit := TestCrash(ShotX[1]  , ShotY[1]  , 28, 20, x[2]+2, y[2], 25, 18);
  end;
  if SwordHit then
    if Energy[2] > SprInfo[Art[1]].FirePower then begin
      Stripe(353, 350-Energy[2], 7, SprInfo[Art[1]].FirePower, 0);
      Stripe(353, 700-Energy[2], 7, SprInfo[Art[1]].FirePower, 0);
      Dec(Energy[2], SprInfo[Art[1]].FirePower);
    end else if Energy[2] > 0 then begin
          Stripe(353, 0, 7, 350, 0);
          Stripe(353, 350, 7, 350, 0);
          Energy[2] := 0;
        end;
  Delay(10);
  SetScrPos(0, 350);
  SwordBkgr[1]^.RestoreBkgr(ShotX[1], ShotY[1]);
  BackGr[1]^.RestoreBkgr(OldX[1], OldY[1]);
  BackGr[3]^.RestoreBkgr(OldX[2], OldY[2]);
  if (d >= 5) then P[1]^.PutSprite(OldX[1], OldY[1], SprInfo[Knight].WalkAnim[4], 1)
    else P[1]^.PutSprite(OldX[1], OldY[1], SprInfo[Knight].WalkAnim[d], 1);
  P[PtrNum]^.PutSprite(OldX[2], OldY[2], SprInfo[Art[2]].WalkAnim[LastDir[2]], 1);
  SetScrPos(0, 0);
  SwordBkgr[2]^.RestoreBkgr(ShotX[1], 350+ShotY[1]);
  BackGr[2]^.RestoreBkgr(OldX[1], 350+OldY[1]);
  BackGr[4]^.RestoreBkgr(OldX[2], 350+OldY[2]);
  if (d >= 5) then P[1]^.PutSprite(OldX[1], 350+OldY[1], SprInfo[Knight].WalkAnim[4], 1)
    else P[1]^.PutSprite(OldX[1], 350+OldY[1], SprInfo[Knight].WalkAnim[d], 1);
  P[PtrNum]^.PutSprite(OldX[2], 350+OldY[2], SprInfo[Art[2]].WalkAnim[LastDir[2]], 1);
  case d of
    3 : FlipXRange(24, 25, P[1]);
    5 : FlipXRange(30, 31, P[1]);
    7 : FlipXRange(32, 33, P[1]);
  end;
  ShotTimer[1] := 0;
end;

procedure Sword2(d : Byte);
var a, b     : Byte;
    SwordHit : Boolean;
begin
  case d of
    1 : begin
          a := 75; b := 76;
          ShotX[2] := x[2]; ShotY[2] := y[2]-20;
        end;
    2 : begin
          a := 77; b := 76;
          P[1]^.FlipSpriteY(76);
          ShotX[2] := x[2]; ShotY[2] := y[2]+20;
        end;
    3 : begin
          a := 73; b := 74;
          FlipXRange(73, 74, P[1]);
          ShotX[2] := x[2]-28; ShotY[2] := y[2];
        end;
    4 : begin
          a := 73; b := 74;
          ShotX[2] := x[2]+28; ShotY[2] := y[2];
        end;
    5 : begin
          a := 78; b := 79;
          FlipXRange(78, 79, P[1]);
          ShotX[2] := x[2]-28; ShotY[2] := y[2]-20;
        end;
    6 : begin
          a := 78; b := 79;
          ShotX[2] := x[2]+28; ShotY[2] := y[2]-20;
        end;
    7 : begin
          a := 80; b := 81;
          FlipXRange(80, 81, P[1]);
          ShotX[2] := x[2]-28; ShotY[2] := y[2]+20;
        end;
    8 : begin
          a := 80; b := 81;
          ShotX[2] := x[2]+28; ShotY[2] := y[2]+20;
        end;
  end;
  ShotDir[2] := d;
  ShotOldX[2] := ShotX[2]; ShotOldY[2] := ShotY[2];
  SetScrPos(0, 350);
  BackGr[1]^.RestoreBkgr(OldX[1], OldY[1]);
  BackGr[3]^.RestoreBkgr(OldX[2], OldY[2]);
  SwordBkgr[3]^.SaveBkgr(ShotX[2], ShotY[2]);
  P[1]^.PutSprite(OldX[1], OldY[1], SprInfo[Art[1]].WalkAnim[LastDir[1]], 1);
  P[1]^.PutSprite(OldX[2], OldY[2], a, 1);
  P[1]^.PutSprite(ShotX[2], ShotY[2], b, 1);
  SetScrPos(0, 0);
  BackGr[2]^.RestoreBkgr(OldX[1], 350+OldY[1]);
  BackGr[4]^.RestoreBkgr(OldX[2], 350+OldY[2]);
  SwordBkgr[4]^.SaveBkgr(ShotX[2], 350+ShotY[2]);
  P[1]^.PutSprite(OldX[1], 350+OldY[1], SprInfo[Art[1]].WalkAnim[LastDir[1]], 1);
  P[1]^.PutSprite(OldX[2], 350+OldY[2], a, 1);
  P[1]^.PutSprite(ShotX[2], 350+ShotY[2], b, 1);
  SwordHit := False;
  case d of
    1, 2       : SwordHit := TestCrash(ShotX[2]+8, ShotY[2]  , 12, 20, x[1]+8, y[1], 12, 16);
    3, 4       : SwordHit := TestCrash(ShotX[2]  , ShotY[2]+5, 28, 20, x[1]+8, y[1], 12, 16);
    5, 6, 7, 8 : SwordHit := TestCrash(ShotX[2]  , ShotY[2]  , 28, 20, x[1]+2, y[1], 25, 18);
  end;
  if SwordHit then
    if Energy[1] > SprInfo[Art[2]].FirePower then begin
      Stripe(0, 350-Energy[1], 7, SprInfo[Art[2]].FirePower, 0);
      Stripe(0, 700-Energy[1], 7, SprInfo[Art[2]].FirePower, 0);
      Dec(Energy[1], SprInfo[Art[2]].FirePower);
    end else if Energy[1] > 0 then begin
          Stripe(0, 0, 7, 350, 0);
          Stripe(0, 350, 7, 350, 0);
          Energy[1] := 0;
        end;
  Delay(10);
  SetScrPos(0, 350);
  SwordBkgr[3]^.RestoreBkgr(ShotX[2], ShotY[2]);
  BackGr[3]^.RestoreBkgr(OldX[2], OldY[2]);
  BackGr[1]^.RestoreBkgr(OldX[1], OldY[1]);
  if (d >= 5) then P[1]^.PutSprite(OldX[2], OldY[2], SprInfo[Goblin].WalkAnim[4], 1)
    else P[1]^.PutSprite(OldX[2], OldY[2], SprInfo[Goblin].WalkAnim[d], 1);
  P[1]^.PutSprite(OldX[1], OldY[1], SprInfo[Art[1]].WalkAnim[LastDir[1]], 1);
  SetScrPos(0, 0);
  SwordBkgr[4]^.RestoreBkgr(ShotX[2], 350+ShotY[2]);
  BackGr[4]^.RestoreBkgr(OldX[2], 350+OldY[2]);
  BackGr[2]^.RestoreBkgr(OldX[1], 350+OldY[1]);
  if (d >= 5) then P[1]^.PutSprite(OldX[2], 350+OldY[2], SprInfo[Goblin].WalkAnim[4], 1)
    else P[1]^.PutSprite(OldX[2], 350+OldY[2], SprInfo[Goblin].WalkAnim[d], 1);
  P[1]^.PutSprite(OldX[1], 350+OldY[1], SprInfo[Art[1]].WalkAnim[LastDir[1]], 1);
  case d of
    2 : P[1]^.FlipSpriteY(76);
    3 : FlipXRange(73, 74, P[1]);
    5 : FlipXRange(78, 79, P[1]);
    7 : FlipXRange(80, 81, P[1]);
  end;
  ShotTimer[2] := 0;
end;

procedure SuckEnergy;
begin
  if Energy[1] > SprInfo[Art[2]].FirePower then begin
    Stripe(0, 350-Energy[1], 7, SprInfo[Art[2]].FirePower, 0);
    Stripe(0, 700-Energy[1], 7, SprInfo[Art[2]].FirePower, 0);
    Dec(Energy[1], SprInfo[Art[2]].FirePower);
  end else if Energy[1] > 0 then begin
        Stripe(0, 0, 7, 350, 0);
        Stripe(0, 350, 7, 350, 0);
        Energy[1] := 0;
      end;
  ShotTimer[2] := 0;
end;

procedure Shoot(PlNum, d : Byte);
var PtrNum, i, j : Byte;
begin
  if Shooting[PlNum] or (ShotTimer[PlNum] < SprInfo[Art[PlNum]].Timer) or
     ((y[PlNum] < 10) and ((d = 1) or (d = 5) or (d = 6))) or
     ((y[PlNum] > 320) and ((d = 2) or (d = 7) or (d = 8))) or
     ((x[PlNum] < 20) and ((d = 3) or (d = 5) or (d = 7))) or
     ((x[PlNum] > 310) and ((d = 4) or (d = 6) or (d = 8))) then Exit;

  if (Art[1] = Knight) and (PlNum = 1) then begin Sword1(d); Exit; end;
  if (Art[2] = Goblin) and (PlNum = 2) then begin Sword2(d); Exit; end;
  if (Art[2] = Siren) and (PlNum = 2) then begin SuckEnergy; Exit; end;

  j := 0;
  if Art[2] >= 96 then begin PtrNum := 2; j := 0; end
    else PtrNum := 1;

  i := d;
  if (i = 5) or (i = 7) then i := 3;
  if (i = 6) or (i = 8) then i := 4;

  if ((Art[1] = Phoenix) or (Art[1] = Unicorn)) and (PlNum = 1) then
    if (i = 1) and FlippedUD[1] then
      FlipYRange(SprInfo[Art[1]].WalkAnim[1], SprInfo[Art[PlNum]].WalkAnim[2], P[1])
    else if (i = 2) and (not FlippedUD[1]) then
           FlipYRange(SprInfo[Art[1]].WalkAnim[1], SprInfo[Art[PlNum]].WalkAnim[2], P[1]);

  if (i = 3) and (not Flipped[PlNum]) then
    if PlNum = 1 then
      FlipXRange(SprInfo[Art[PlNum]].WalkAnim[3], SprInfo[Art[PlNum]].WalkAnim[3]+1, P[1])
    else FlipXRange(SprInfo[Art[PlNum]].WalkAnim[3]-j, SprInfo[Art[PlNum]].WalkAnim[3]+1-j, P[PtrNum]);
  if (i = 4) and Flipped[PlNum] then
    if PlNum = 1 then
      FlipXRange(SprInfo[Art[PlNum]].WalkAnim[4], SprInfo[Art[PlNum]].WalkAnim[4]+1, P[1])
    else FlipXRange(SprInfo[Art[PlNum]].WalkAnim[4]-j, SprInfo[Art[PlNum]].WalkAnim[4]+1-j, P[PtrNum]);

  BackGr[2]^.RestoreBkgr(OldX[1], 350+OldY[1]);
  BackGr[4]^.RestoreBkgr(OldX[2], 350+OldY[2]);

  if PlNum = 1 then begin
    P[1]^.PutSprite(OldX[PlNum], 350+OldY[PlNum], SprInfo[Art[PlNum]].WalkAnim[i], 1);
    P[PtrNum]^.PutSprite(OldX[2], 350+OldY[2], SprInfo[Art[2]].WalkAnim[LastDir[2]]-j, 1);
  end else begin
    P[PtrNum]^.PutSprite(OldX[2], 350+OldY[2], SprInfo[Art[2]].WalkAnim[i]-j, 1);
    P[1]^.PutSprite(OldX[1], 350+OldY[1], SprInfo[Art[1]].WalkAnim[LastDir[1]], 1);
  end;

  SetScrPos(0, 350);

  BackGr[1]^.RestoreBkgr(OldX[1], OldY[1]);
  BackGr[3]^.RestoreBkgr(OldX[2], OldY[2]);

  if PlNum = 1 then begin
    P[1]^.PutSprite(OldX[PlNum], OldY[PlNum], SprInfo[Art[PlNum]].WalkAnim[i], 1);
    P[PtrNum]^.PutSprite(OldX[2], OldY[2], SprInfo[Art[2]].WalkAnim[LastDir[2]]-j, 1);
  end else begin
    P[PtrNum]^.PutSprite(OldX[2], OldY[2], SprInfo[Art[2]].WalkAnim[i]-j, 1);
    P[1]^.PutSprite(OldX[1], OldY[1], SprInfo[Art[1]].WalkAnim[LastDir[1]], 1);
  end;

  if ((Art[1] = Phoenix) or (Art[1] = Unicorn)) and (PlNum = 1) then
    if (i = 1) and FlippedUD[1] then
      FlipYRange(SprInfo[Art[1]].WalkAnim[1], SprInfo[Art[PlNum]].WalkAnim[2], P[1])
    else if (i = 2) and (not FlippedUD[1]) then
           FlipYRange(SprInfo[Art[1]].WalkAnim[1], SprInfo[Art[PlNum]].WalkAnim[2], P[1]);

  if (i = 3) and (not Flipped[PlNum]) then
    if PlNum = 1 then
      FlipXRange(SprInfo[Art[PlNum]].WalkAnim[3], SprInfo[Art[PlNum]].WalkAnim[3]+1, P[1])
    else FlipXRange(SprInfo[Art[PlNum]].WalkAnim[3]-j, SprInfo[Art[PlNum]].WalkAnim[3]+1-j, P[PtrNum]);
  if (i = 4) and Flipped[PlNum] then
    if PlNum = 1 then
      FlipXRange(SprInfo[Art[PlNum]].WalkAnim[4], SprInfo[Art[PlNum]].WalkAnim[4]+1, P[1])
    else FlipXRange(SprInfo[Art[PlNum]].WalkAnim[4]-j, SprInfo[Art[PlNum]].WalkAnim[4]+1-j, P[PtrNum]);

  SetScrPos(0, 0);

  case d of
    1 : begin ShotX[PlNum] := x[PlNum]+8; ShotY[PlNum] := y[PlNum]-10; end;
    2 : begin ShotX[PlNum] := x[PlNum]+8; ShotY[PlNum] := y[PlNum]+22; end;
    3 : begin ShotX[PlNum] := x[PlNum]-13; ShotY[PlNum] := y[PlNum]+7; end;
    4 : begin ShotX[PlNum] := x[PlNum]+30; ShotY[PlNum] := y[PlNum]+7; end;
    5 : begin ShotX[PlNum] := x[PlNum]-13; ShotY[PlNum] := y[PlNum]-10; end;
    6 : begin ShotX[PlNum] := x[PlNum]+30; ShotY[PlNum] := y[PlNum]-10; end;
    7 : begin ShotX[PlNum] := x[PlNum]-13; ShotY[PlNum] := y[PlNum]+22; end;
    8 : begin ShotX[PlNum] := x[PlNum]+30; ShotY[PlNum] := y[PlNum]+22; end;
  end;
  ShotDir[PlNum] := d;
  ShotOldX[PlNum] := ShotX[PlNum]; ShotOldY[PlNum] := ShotY[PlNum];
  if PlNum = 1 then begin
    ShotBkgr[1]^.SaveBkgr(ShotX[1], ShotY[1]);
    ShotBkgr[2]^.SaveBkgr(ShotX[1], 350+ShotY[1]);
  end else begin
    ShotBkgr[3]^.SaveBkgr(ShotX[2], ShotY[2]);
    ShotBkgr[4]^.SaveBkgr(ShotX[2], 350+ShotY[2]);
  end;
  Shooting[PlNum] := True;
  ShotTimer[PlNum] := 0;
end;

function MayShoot(n : Byte) : Boolean;
begin
  MayShoot := True;
  case ShotDir[n] of
    1 : if (GetPixel(ShotOldX[n]+1, ShotOldY[n]-1) < 127) or
           (GetPixel(ShotOldX[n]+11, ShotOldY[n]-1) < 127) or
           (ShotOldY[n] <= ShotSpeed[n]) then MayShoot := False;
    2 : if (GetPixel(ShotOldX[n]+1, ShotOldY[n]+8) < 127) or
           (GetPixel(ShotOldX[n]+11, ShotOldY[n]+8) < 127) or
           (ShotOldY[n] >= 349-ShotSpeed[n]) then MayShoot := False;
    3 : if (GetPixel(ShotOldX[n]-1, ShotOldY[n]+1) < 127) or
           (GetPixel(ShotOldX[n]-1, ShotOldY[n]+6) < 127) or
           (ShotOldX[n] <= ShotSpeed[n]) then MayShoot := False;
    4 : if (GetPixel(ShotOldX[n]+11, ShotOldY[n]+1) < 127) or
           (GetPixel(ShotOldX[n]+11, ShotOldY[n]+6) < 127) or
           (ShotOldX[n] >= 359-ShotSpeed[n]) then MayShoot := False;
    5 : if (GetPixel(ShotOldX[n]-1, ShotOldY[n]+1) < 127) or
           (GetPixel(ShotOldX[n]-1, ShotOldY[n]+6) < 127) or
           (ShotOldX[n] <= ShotSpeed[n]) or
           (GetPixel(ShotOldX[n]+1, ShotOldY[n]-1) < 127) or
           (GetPixel(ShotOldX[n]+11, ShotOldY[n]-1) < 127) or
           (ShotOldY[n] <= ShotSpeed[n]) then MayShoot := False;
    6 : if (GetPixel(ShotOldX[n]+11, ShotOldY[n]+1) < 127) or
           (GetPixel(ShotOldX[n]+11, ShotOldY[n]+6) < 127) or
           (ShotOldX[n] >= 359-ShotSpeed[n]) or
           (GetPixel(ShotOldX[n]+1, ShotOldY[n]-1) < 127) or
           (GetPixel(ShotOldX[n]+11, ShotOldY[n]-1) < 127) or
           (ShotOldY[n] <= ShotSpeed[n]) then MayShoot := False;
    7 : if (GetPixel(ShotOldX[n]-1, ShotOldY[n]+1) < 127) or
           (GetPixel(ShotOldX[n]-1, ShotOldY[n]+6) < 127) or
           (ShotOldX[n] <= ShotSpeed[n]) or
           (GetPixel(ShotOldX[n]+1, ShotOldY[n]+8) < 127) or
           (GetPixel(ShotOldX[n]+11, ShotOldY[n]+8) < 127) or
           (ShotOldY[n] >= 349-ShotSpeed[n]) then MayShoot := False;
    8 : if (GetPixel(ShotOldX[n]+12, ShotOldY[n]+1) < 127) or
           (GetPixel(ShotOldX[n]+12, ShotOldY[n]+6) < 127) or
           (ShotOldX[n] >= 359-ShotSpeed[n]) or
           (GetPixel(ShotOldX[n]+1, ShotOldY[n]+8) < 127) or
           (GetPixel(ShotOldX[n]+12, ShotOldY[n]+8) < 127) or
           (ShotOldY[n] >= 349-ShotSpeed[n]) then MayShoot := False;
  end;
end;

procedure CheckKeys;
begin

  { ==================== LIGHT =================== }

  if Key[ScUp] and (not Key[ScLeft]) and (not Key[ScRight]) and
                   (not Key[ScHome]) and (not Key[ScPgUp]) then
    if Key[ScSpace] then Shoot(1, 1)
      else if MayGo(1, 1) then SetNewPos(1, 1);
  if Key[ScDown] and (not Key[ScLeft]) and (not Key[ScRight]) and
                     (not Key[ScEnd]) and (not Key[ScPgDn]) then
    if Key[ScSpace] then Shoot(1, 2)
      else if MayGo(1, 2) then SetNewPos(1, 2);
  if Key[ScLeft] and (not Key[ScUp]) and (not Key[ScDown]) and
                     (not Key[ScHome]) and (not Key[ScEnd]) then
    if Key[ScSpace] then Shoot(1, 3)
      else if MayGo(1, 3) then SetNewPos(1, 3);
  if Key[ScRight] and (not Key[ScUp]) and (not Key[ScDown]) and
                      (not Key[ScPgUp]) and (not Key[ScPgDn]) then
    if Key[ScSpace] then Shoot(1, 4)
      else if MayGo(1, 4) then SetNewPos(1, 4);
  if ((Key[ScUp] and Key[ScLeft]) or Key[ScHome]) and
     (not Key[ScPgUp]) and (not Key[ScEnd]) then
    if Key[ScSpace] then Shoot(1, 5)
      else if MayGo(1, 5) then SetNewPos(1, 5)
             else if MayGo(1, 1) then SetNewPos(1, 1)
                    else if MayGo(1, 3) then SetNewPos(1, 3);
  if ((Key[ScUp] and Key[ScRight]) or Key[ScPgUp]) and
     (not Key[ScHome]) and (not Key[ScPgDn]) then
    if Key[ScSpace] then Shoot(1, 6)
      else if MayGo(1, 6) then SetNewPos(1, 6)
             else if MayGo(1, 1) then SetNewPos(1, 1)
                    else if MayGo(1, 4) then SetNewPos(1, 4);
  if ((Key[ScDown] and Key[ScLeft]) or Key[ScEnd]) and
     (not Key[ScPgDn]) and (not Key[ScHome]) then
    if Key[ScSpace] then Shoot(1, 7)
      else if MayGo(1, 7) then SetNewPos(1, 7)
             else if MayGo(1, 2) then SetNewPos(1, 2)
                    else if MayGo(1, 3) then SetNewPos(1, 3);
  if ((Key[ScDown] and Key[ScRight]) or Key[ScPgDn]) and
     (not Key[ScEnd]) and (not Key[ScPgUp]) then
    if Key[ScSpace] then Shoot(1, 8)
      else if MayGo(1, 8) then SetNewPos(1, 8)
             else if MayGo(1, 2) then SetNewPos(1, 2)
                    else if MayGo(1, 4) then SetNewPos(1, 4);

  { ===================== DARK ===================== }

  if Key[ScW] and (not Key[ScA]) and (not Key[ScD]) and
                  (not Key[ScQ]) and (not Key[ScE]) then
    if Key[ScTab] or Key[ScS] then Shoot(2, 1)
      else if MayGo(2, 1) then SetNewPos(2, 1);
  if Key[ScX] and (not Key[ScA]) and (not Key[ScD]) and
                  (not Key[ScY]) and (not Key[ScC])then
    if Key[ScTab] or Key[ScS] then Shoot(2, 2)
      else if MayGo(2, 2) then SetNewPos(2, 2);
  if Key[ScA] and (not Key[ScW]) and (not Key[ScX]) and
                  (not Key[ScQ]) and (not Key[ScY]) then
    if Key[ScTab] or Key[ScS] then Shoot(2, 3)
      else if MayGo(2, 3) then SetNewPos(2, 3);
  if Key[ScD] and (not Key[ScW]) and (not Key[ScX]) and
                  (not Key[ScE]) and (not Key[ScX]) then
    if Key[ScTab] or Key[ScS] then Shoot(2, 4)
      else if MayGo(2, 4) then SetNewPos(2, 4);
  if ((Key[ScW] and Key[ScA]) or Key[ScQ]) and
     (not Key[ScE]) and (not Key[ScY]) then
    if Key[ScTab] or Key[ScS] then Shoot(2, 5)
      else if MayGo(2, 5) then SetNewPos(2, 5)
             else if MayGo(2, 1) then SetNewPos(2, 1)
                    else if MayGo(2, 3) then SetNewPos(2, 3);
  if ((Key[ScW] and Key[ScD]) or Key[ScE]) and
     (not Key[ScQ]) and (not Key[ScX]) then
    if Key[ScTab] or Key[ScS] then Shoot(2, 6)
      else if MayGo(2, 6) then SetNewPos(2, 6)
             else if MayGo(2, 1) then SetNewPos(2, 1)
                    else if MayGo(2, 4) then SetNewPos(2, 4);
  if ((Key[ScX] and Key[ScA]) or Key[ScY]) and
     (not Key[ScC]) and (not Key[ScQ]) then
    if Key[ScTab] or Key[ScS] then Shoot(2, 7)
      else if MayGo(2, 7) then SetNewPos(2, 7)
             else if MayGo(2, 2) then SetNewPos(2, 2)
                    else if MayGo(2, 3) then SetNewPos(2, 3);
  if ((Key[ScX] and Key[ScD]) or Key[ScC]) and
     (not Key[ScY]) and (not Key[ScE]) then
    if Key[ScTab] or Key[ScS] then Shoot(2, 8)
      else if MayGo(2, 8) then SetNewPos(2, 8)
             else if MayGo(2, 2) then SetNewPos(2, 2)
                    else if MayGo(2, 4) then SetNewPos(2, 4);
end;

function Hit(n : Byte) : Boolean;
var i : Byte;
begin
  if n = 1 then i := 2
    else i := 1;
  Hit := TestCrash(ShotX[n], ShotY[n], 12, 7, x[i], y[i], 28, 20);
end;

begin
  New(B, InitBkgr(80, 200));
  B^.SaveBkgr(0, 0);
  Blend(False);
  ClearNScreen;
  InitNGraph(N360x350, 90);
  SetOverscan(0);
  ShowPCX(StartDir+'\GRAPHX\FIGHT.GFX', 0, 0, Random(3)+1);
  RMove(0, 0, 360, 350, 0, 350);
  SetUpFighters(Art1, Art2);
  Blend(True);
  repeat
    OldX[1] := x[1]; OldY[1] := y[1];
    OldX[2] := x[2]; OldY[2] := y[2];
    ShotOldX[1] := ShotX[1]; ShotOldY[1] := ShotY[1];
    ShotOldX[2] := ShotX[2]; ShotOldY[2] := ShotY[2];

    for i := 1 to 2 do
      if (Shooting[i]) and MayShoot(i) and (not Hit(i)) and (Art[i] <> Knight) then begin
        case ShotDir[i] of
          1 : Dec(ShotY[i], ShotSpeed[i]);
          2 : Inc(ShotY[i], ShotSpeed[i]);
          3 : Dec(ShotX[i], ShotSpeed[i]);
          4 : Inc(ShotX[i], ShotSpeed[i]);
          5 : begin Dec(ShotX[i], ShotSpeed[i]); Dec(ShotY[i], ShotSpeed[i]); end;
          6 : begin Inc(ShotX[i], ShotSpeed[i]); Dec(ShotY[i], ShotSpeed[i]); end;
          7 : begin Dec(ShotX[i], ShotSpeed[i]); Inc(ShotY[i], ShotSpeed[i]); end;
          8 : begin Inc(ShotX[i], ShotSpeed[i]); Inc(ShotY[i], ShotSpeed[i]); end;
        end;
      end else begin
            if Hit(i) and Shooting[i] then begin
              if i = 1 then if Energy[2] > SprInfo[Art[1]].FirePower then begin
                              Stripe(353, 350-Energy[2], 7, SprInfo[Art[1]].FirePower, 0);
                              Stripe(353, 700-Energy[2], 7, SprInfo[Art[1]].FirePower, 0);
                              Dec(Energy[2], SprInfo[Art[1]].FirePower);
                            end else begin
                                  Stripe(353, 0, 7, 350, 0);
                                  Stripe(353, 350, 7, 350, 0);
                                  Energy[2] := 0;
                                end;
              if i = 2 then if Energy[1] > SprInfo[Art[2]].FirePower then begin
                              Stripe(0, 350-Energy[1], 7, SprInfo[Art[2]].FirePower, 0);
                              Stripe(0, 700-Energy[1], 7, SprInfo[Art[2]].FirePower, 0);
                              Dec(Energy[1], SprInfo[Art[2]].FirePower);
                            end else begin
                                  Stripe(0, 0, 7, 350, 0);
                                  Stripe(0, 350, 7, 350, 0);
                                  Energy[1] := 0;
                                end;
            end;
            if Shooting[i] and (Art[i] <> Knight) then begin
              Shooting[i] := False;
              if i = 1 then begin
                ShotBkgr[1]^.RestoreBkgr(ShotOldX[1], ShotOldY[1]);
                ShotBkgr[2]^.RestoreBkgr(ShotOldX[1], 350+ShotOldY[1]);
              end else begin
                ShotBkgr[3]^.RestoreBkgr(ShotOldX[2], ShotOldY[2]);
                ShotBkgr[4]^.RestoreBkgr(ShotOldX[2], 350+ShotOldY[2]);
              end;
            end;
          end;

    CheckKeys;

    for i := 1 to 2 do if Moved[i] >= 4 then begin
      Moved[i] := 0;
      if LastAnim[i] = 0 then LastAnim[i] := 1
        else LastAnim[i] := 0;
    end;

    if Shooting[1] then if Art[1] <> Knight then ShotBkgr[2]^.RestoreBkgr(ShotOldX[1], 350+ShotOldY[1]);
    if Shooting[2] then ShotBkgr[4]^.RestoreBkgr(ShotOldX[2], 350+ShotOldY[2]);
    if Shooting[1] then if Art[1] <> Knight then ShotBkgr[2]^.SaveBkgr(ShotX[1], 350+ShotY[1]);
    if Shooting[2] then ShotBkgr[4]^.SaveBkgr(ShotX[2], 350+ShotY[2]);

    ChangeCheck := Changed(1) or Changed(2);

    if ChangeCheck then begin
      BackGr[2]^.RestoreBkgr(OldX[1], 350+OldY[1]);
      BackGr[4]^.RestoreBkgr(OldX[2], 350+OldY[2]);
      BackGr[2]^.SaveBkgr(x[1], 350+y[1]);
      BackGr[4]^.SaveBkgr(x[2], 350+y[2]);
      P[1]^.PutSprite(x[1], 350+y[1], SprInfo[Art[1]].WalkAnim[Dir[1]]+LastAnim[1], 1);
      P[PtrNumber]^.PutSprite(x[2], 350+y[2], SprInfo[Art[2]].WalkAnim[Dir[2]]+LastAnim[2], 1);
    end;

    if Shooting[1] and (Art[1] <> Knight) then begin
      i := SprInfo[Art[1]].ShotAnim+ShotDir[1]-1;
      if (ShotDir[1] = 5) or (ShotDir[1] = 7) then i := SprInfo[Art[1]].ShotAnim+2;
      if (ShotDir[1] = 6) or (ShotDir[1] = 8) then i := SprInfo[Art[1]].ShotAnim+3;
      S^.PutSprite(ShotX[1], 350+ShotY[1], i, 1);
    end;
    if Shooting[2] then begin
      i := SprInfo[Art[2]].ShotAnim+ShotDir[2]-1;
      if (ShotDir[2] = 5) or (ShotDir[2] = 7) then i := SprInfo[Art[2]].ShotAnim+2;
      if (ShotDir[2] = 6) or (ShotDir[2] = 8) then i := SprInfo[Art[2]].ShotAnim+3;
      S^.PutSprite(ShotX[2], 350+ShotY[2], i, 1);
    end;

    SetScrPos(0, 350);

    if Shooting[1] then if Art[1] <> Knight then ShotBkgr[1]^.RestoreBkgr(ShotOldX[1], ShotOldY[1]);
    if Shooting[2] then ShotBkgr[3]^.RestoreBkgr(ShotOldX[2], ShotOldY[2]);
    if Shooting[1] then if Art[1] <> Knight then ShotBkgr[1]^.SaveBkgr(ShotX[1], ShotY[1]);
    if Shooting[2] then ShotBkgr[3]^.SaveBkgr(ShotX[2], ShotY[2]);

    if ChangeCheck then begin
      BackGr[1]^.RestoreBkgr(OldX[1], OldY[1]);
      BackGr[3]^.RestoreBkgr(OldX[2], OldY[2]);
      BackGr[1]^.SaveBkgr(x[1], y[1]);
      BackGr[3]^.SaveBkgr(x[2], y[2]);
      P[1]^.PutSprite(x[1], y[1], SprInfo[Art[1]].WalkAnim[Dir[1]]+LastAnim[1], 1);
      P[PtrNumber]^.PutSprite(x[2], y[2], SprInfo[Art[2]].WalkAnim[Dir[2]]+LastAnim[2], 1);
    end;

    if Shooting[1] and (Art[1] <> Knight) then begin
      i := SprInfo[Art[1]].ShotAnim+ShotDir[1]-1;
      if (ShotDir[1] = 5) or (ShotDir[1] = 7) then i := SprInfo[Art[1]].ShotAnim+2;
      if (ShotDir[1] = 6) or (ShotDir[1] = 8) then i := SprInfo[Art[1]].ShotAnim+3;
      S^.PutSprite(ShotX[1], ShotY[1], i, 1);
    end;

    if Shooting[2] then begin
      i := SprInfo[Art[2]].ShotAnim+ShotDir[2]-1;
      if (ShotDir[2] = 5) or (ShotDir[2] = 7) then i := SprInfo[Art[2]].ShotAnim+2;
      if (ShotDir[2] = 6) or (ShotDir[2] = 8) then i := SprInfo[Art[2]].ShotAnim+3;
      S^.PutSprite(ShotX[2], ShotY[2], i, 1);
    end;

    SetScrPos(0, 0);
  until (Energy[1] = 0) or (Energy[2] = 0);
  if (Energy[1] = 0) then Winner := Art[2] else Winner := Art[1];
  if Flipped[1] then FlipXRange(Art[1], Art[1]+1, P[1]);
  if Flipped[2] then if PtrNumber = 1 then FlipXRange(Art[2], Art[2]+1, P[1])
                       else FlipXRange(Art[2]-96, Art[2]-95, P[2]);
  if FlippedUD[1] then FlipYRange(Art[1]+2, Art[1]+3, P[1]);
  if Art[2] >= 96 then
    for i := 1 to 4 do Inc(SprInfo[Art[2]].WalkAnim[i], 96);
  CheckExit := False;
  DoExit := False;
  Blend(False);
  ClearNScreen;
  InitNGraph(N320x200, 80);
  Screen(0);
  SetVGAPal(ArchonPal, 0, 256);
  B^.RestoreBkgr(0, 0);
  Dispose(B, DisposeBkgr);
  for i := 1 to 4 do begin
    Dispose(BackGr[i], DisposeBkgr);
    Dispose(ShotBkGr[i], DisposeBkgr);
    Dispose(SwordBkGr[i], DisposeBkgr);
  end;
  RMove(0, 0, 320, 200, 0, 200);
  RMove(0, 0, 320, 200, 0, 400);
  for i := 0 to 8 do
    for j := 0 to 8 do if ScreenSet[j, i] <> Neut then
                         E^.PutSprite(34+i*28, 402+j*20, ScreenSet[j, i], 0)
                       else E^.PutSprite(34+i*28, 402+j*20, NeutralColor, 0);
  E^.PutSprite(146, 402, 6, 1);
  E^.PutSprite(146, 482, 6, 1);
  E^.PutSprite(146, 562, 6, 1);
  E^.PutSprite(34, 482, 6, 1);
  E^.PutSprite(258, 482, 6, 1);
  i := 0;
  if Winner = Art[2] then begin
    repeat
      Inc(i);
    until (KilledItems[1, i] = 255) or (KilledItems[1, i] = Art[1]);
    KilledItems[1, i] := Art[1];
    P[1]^.PutSprite(2, 2+(i-1)*20, Art[1], 1);
    P[1]^.PutSprite(2, 202+(i-1)*20, Art[1], 1);
    P[1]^.PutSprite(2, 402+(i-1)*20, Art[1], 1);
  end else begin
    repeat
      Inc(i);
    until (KilledItems[2, i] = 255) or (KilledItems[2, i] = Art[2]);
    KilledItems[2, i] := Art[2];
    if Art[2] >= 96 then begin
      P[2]^.FlipSpriteX(Art[2]-96);
      P[2]^.PutSprite(290, 2+(i-1)*20, Art[2]-96, 1);
      P[2]^.PutSprite(290, 202+(i-1)*20, Art[2]-96, 1);
      P[2]^.PutSprite(290, 402+(i-1)*20, Art[2]-96, 1);
      P[2]^.FlipSpriteX(Art[2]-96);
    end else begin
      P[1]^.FlipSpriteX(Art[2]);
      P[1]^.PutSprite(290, 2+(i-1)*20, Art[2], 1);
      P[1]^.PutSprite(290, 202+(i-1)*20, Art[2], 1);
      P[1]^.PutSprite(290, 402+(i-1)*20, Art[2], 1);
      P[1]^.FlipSpriteX(Art[2]);
    end;
  end;
  Blend(True);
end;

procedure PutRestAnim;
begin
  if (RestAnim >= 60) or (PtrNumber = 2) then P[PtrNumber]^.FlipSpriteX(RestAnim);
  SetScrPos(0, 200);
  Bg[1]^.RestoreBkgr(OldX, OldY);
  P[PtrNumber]^.PutSprite(OldX, OldY, RestAnim, 1);
  SetScrPos(0, 0);
  Bg[2]^.RestoreBkgr(OldX, 200+OldY);
  P[PtrNumber]^.PutSprite(OldX, 200+OldY, RestAnim, 1);
  if (RestAnim >= 60) or (PtrNumber = 2) then P[PtrNumber]^.FlipSpriteX(RestAnim);
end;

procedure InitFight(var Art, x, y, Winner : Word);
var Tmp, FirePower : Word;
    Impr           : Boolean;
begin
  Impr := Imprisoned[x, y];
  if Pl1Turn then begin
    Tmp := SprInfo[Art].FirePower;
    FirePower := 350 div Tmp;
    case ScreenSet[y, x] of
      White   : Dec(FirePower, 2);
      Cyan    : Dec(FirePower, 1);
      Blue    : Inc(FirePower, 1);
      Black   : Inc(FirePower, 2);
    end;
    SprInfo[Art].FirePower := 353 div FirePower;
    Fight(Art, ItemSet[y, x], Winner);
    SprInfo[Art].FirePower := Tmp;
  end else begin
    Tmp := SprInfo[ItemSet[y, x]].FirePower;
    FirePower := 350 div Tmp;
    case ScreenSet[y, x] of
      White   : Dec(FirePower, 2);
      Cyan    : Dec(FirePower, 1);
      Blue    : Inc(FirePower, 1);
      Black   : Inc(FirePower, 2);
    end;
    SprInfo[ItemSet[y, x]].FirePower := 353 div FirePower;
    Fight(ItemSet[y, x], Art, Winner);
    SprInfo[ItemSet[y, x]].FirePower := Tmp;
  end;
  if Winner = Art then Imprisoned[x, y] := False;
  ItemSet[y, x] := Winner;
  RestAnim := SprInfo[Winner].RestAnim;
  if Winner >= 96 then begin
    PtrNumber := 2;
    Dec(RestAnim, 96);
  end else PtrNumber := 1;
  Bg[1]^.SaveBkgr(34+x*28, 402+y*20);
  Bg[2]^.SaveBkgr(34+x*28, 402+y*20);
  PutRestAnim;
end;

function MoveSprite : Boolean;
var x, y, StartX, StartY,
    Range                : Byte;
    Art                  : Word;
    Ch                   : Char;
    U, D, L, R, F1, F2,
    FirePower, Tmp,
    x2, y2, OldX2, OldY2 : Word;
    Fly                  : Boolean;

function MayGo(d, x, y, Range : Byte; Fly : Boolean) : Boolean;
begin
  if Pl1Turn then case d of
    1 : MayGo := (y > 0) and (y > StartY-Range) and
                 ((ItemSet[y-1, x] = 255) or
                 ((ItemSet[y-1, x] >= 60) and (ItemSet[y, x] = 255)) or
                 Fly);
    2 : MayGo := (y < 8) and (y < StartY+Range) and
                 ((ItemSet[y+1, x] = 255) or
                 ((ItemSet[y+1, x] >= 60) and (ItemSet[y, x] = 255)) or
                 Fly);
    3 : MayGo := (x > 0) and (x > StartX-Range) and
                 ((ItemSet[y, x-1] = 255) or
                 ((ItemSet[y, x-1] >= 60) and (ItemSet[y, x] = 255)) or
                 Fly);
    4 : MayGo := (x < 8) and (x < StartX+Range) and
                 ((ItemSet[y, x+1] = 255) or
                 ((ItemSet[y, x+1] >= 60) and (ItemSet[y, x] = 255)) or
                 Fly);
  end else case d of
    1 : MayGo := (y > 0) and (y > StartY-Range) and
                 ((ItemSet[y-1, x] = 255) or
                 ((ItemSet[y-1, x] < 60) and (ItemSet[y, x] = 255)) or
                 Fly);
    2 : MayGo := (y < 8) and (y < StartY+Range) and
                 ((ItemSet[y+1, x] = 255) or
                 ((ItemSet[y+1, x] < 60) and (ItemSet[y, x] = 255)) or
                 Fly);
    3 : MayGo := (x > 0) and (x > StartX-Range) and
                 ((ItemSet[y, x-1] = 255) or
                 ((ItemSet[y, x-1] < 60) and (ItemSet[y, x] = 255)) or
                 Fly);
    4 : MayGo := (x < 8) and (x < StartX+Range) and
                 ((ItemSet[y, x+1] = 255) or
                 ((ItemSet[y, x+1] < 60) and (ItemSet[y, x] = 255)) or
                 Fly);
  end;
end;

begin
  if Pl1Turn then begin
    U := ScUp; D := ScDown;
    L := ScLeft; R := ScRight;
    F1 := ScSpace; F2 := ScReturn;
  end else begin
    U := ScW; D := ScX;
    L := ScA; R := ScD;
    F1 := ScS; F2 := ScTab;
  end;
  MoveSprite := False;
  Key[F1] := False; Key[F2] := False;
  x := (CursorX-34) div 28; y := (CursorY-2) div 20;
  OldX := x; OldY := y;
  StartX := x; StartY := y;
  if ItemSet[y, x] = 255 then begin
    WriteMsg('No item!'); Delay(1500); ClearMsg; Exit;
  end;
  if ((ItemSet[y, x] >= 60) and Pl1Turn) or
     ((ItemSet[y, x] <= 59) and (not Pl1Turn)) then begin
    WriteMsg('Enemy item!'); Delay(1500); ClearMsg; Exit;
  end;
  if Imprisoned[x, y] then begin
    WriteMsg('Item is imprisoned!'); Delay(1500); ClearMsg; Exit;
  end;
  case ItemSet[y, x] of
    Wizard      : WriteMsg('Light Wizard - Fly, 3');
    Golem       : WriteMsg('Golem - Walk, 3');
    Phoenix     : WriteMsg('Phoenix - Fly, 5');
    Knight      : WriteMsg('Knight - Walk, 3');
    Unicorn     : WriteMsg('Unicorn - Walk, 4');
    Djinni      : WriteMsg('Djinni - Fly, 4');
    Archer      : WriteMsg('Archer - Walk, 3');
    Valkyrie    : WriteMsg('Valkyrie - Fly, 3');
    Manticore   : WriteMsg('Manticore - Walk, 3');
    Goblin      : WriteMsg('Goblin - Walk, 3');
    Basilisk    : WriteMsg('Basilisk - Walk, 4');
    Troll       : WriteMsg('Troll - Walk, 3');
    Siren       : WriteMsg('Siren - Fly, 3');
    Dragon      : WriteMsg('Dragon - Fly, 5');
    DarkWizard  : WriteMsg('Dark Wizard - Fly, 3');
    FireCritter : WriteMsg('Fire Critter - Fly, 4');
  end;
  HideCursor;
  Art := ItemSet[y, x];
  Fly := SprInfo[Art].Fly;
  Range := SprInfo[Art].Range;
  SprInfo[Art].x := CursorX; SprInfo[Art].y := CursorY;
  FirstRestore := True;
  ItemSet[y, x] := 255;
  Walked := False;
  Winner := Art;
  repeat
    if Key[U] and MayGo(1, x, y, Range, Fly) then begin Dec(y); Walk(Art, 1, 1, False, FirstRestore); end;
    if Key[D] and MayGo(2, x, y, Range, Fly) then begin Inc(y); Walk(Art, 2, 1, False, FirstRestore); end;
    if Key[L] and MayGo(3, x, y, Range, Fly) then begin Dec(x); Walk(Art, 3, 1, False, FirstRestore); end;
    if Key[R] and MayGo(4, x, y, Range, Fly) then begin Inc(x); Walk(Art, 4, 1, False, FirstRestore); end;
  until (Key[F1] or Key[F2]) and ((ItemSet[y, x] = 255) or
        ((ItemSet[y, x] >= 60) and Pl1Turn) or
        ((ItemSet[y, x] <= 59) and (not Pl1Turn)));
  if ItemSet[y, x] <> 255 then begin
    x2 := x; y2 := y;
    InitFight(Art, x2, y2, Winner);
    MoveSprite := True;
    x := x2; y := y2;
  end;
  ItemSet[y, x] := Winner;
  CursorX := SprInfo[Art].x; CursorY := SprInfo[Art].y;
  if (x <> OldX) or (y <> OldY) or Walked then PutRestAnim;
  if Walked then MoveSprite := True;
  ShowCursor;
  while KeyPressed do Ch := ReadKey;
  ClearMsg;
end;

procedure DrawSprites(Page : Byte);
var a, b : Byte;
begin
  for a := 0 to 8 do
    for b := 0 to 8 do if ItemSet[b, a] >= 60 then begin
                         if ItemSet[b, a] >= 96 then begin
                           P[2]^.FlipSpriteX(ItemSet[b, a]-96);
                           P[2]^.PutSprite(34+a*28, Page*200+2+b*20, ItemSet[b, a]-96, 1);
                           P[2]^.FlipSpriteX(ItemSet[b, a]-96);
                         end else begin
                           P[1]^.FlipSpriteX(ItemSet[b, a]);
                           P[1]^.PutSprite(34+a*28, Page*200+2+b*20, ItemSet[b, a], 1);
                           P[1]^.FlipSpriteX(ItemSet[b, a]);
                         end;
                       end else P[1]^.PutSprite(34+a*28, Page*200+2+b*20, ItemSet[b, a], 1);
end;

procedure DrawNeutralColor;
var a, b : Byte;
begin
  SetScrPos(0, 200);
  for a := 0 to 8 do
    for b := 0 to 8 do if ScreenSet[b, a] = Neut then E^.PutSprite(34+a*28, 2+b*20, NeutralColor, 0);
  E^.PutSprite(146, 2, 6, 1);
  E^.PutSprite(146, 82, 6, 1);
  E^.PutSprite(146, 162, 6, 1);
  E^.PutSprite(34, 82, 6, 1);
  E^.PutSprite(258, 82, 6, 1);
  DrawSprites(0);
  SetScrPos(0, 0);
  for a := 0 to 8 do
    for b := 0 to 8 do if ScreenSet[b, a] = Neut then begin
                         E^.PutSprite(34+a*28, 202+b*20, NeutralColor, 0);
                         E^.PutSprite(34+a*28, 402+b*20, NeutralColor, 0);
                       end;
  E^.PutSprite(146, 202, 6, 1);
  E^.PutSprite(146, 282, 6, 1);
  E^.PutSprite(146, 362, 6, 1);
  E^.PutSprite(34, 282, 6, 1);
  E^.PutSprite(258, 282, 6, 1);

  E^.PutSprite(146, 402, 6, 1);
  E^.PutSprite(146, 482, 6, 1);
  E^.PutSprite(146, 562, 6, 1);
  E^.PutSprite(34, 482, 6, 1);
  E^.PutSprite(258, 482, 6, 1);
  DrawSprites(1);
end;

procedure RedrawAll;
var a, b : Byte;
begin
  HideCursor;
  SetScrPos(0, 200);
  XMove(32, 400, 32, 0, 256, 184, 0, 0, 0);
  for a := 0 to 8 do
    for b := 0 to 8 do if ScreenSet[b, a] = Neut then E^.PutSprite(34+a*28, 2+b*20, NeutralColor, 0);
  E^.PutSprite(146, 2, 6, 1);
  E^.PutSprite(146, 82, 6, 1);
  E^.PutSprite(146, 162, 6, 1);
  E^.PutSprite(34, 82, 6, 1);
  E^.PutSprite(258, 82, 6, 1);
  DrawSprites(0);
  SetScrPos(0, 0);
  XMove(32, 400, 32, 200, 256, 184, 0, 0, 0);
  for a := 0 to 8 do
    for b := 0 to 8 do if ScreenSet[b, a] = Neut then begin
                         E^.PutSprite(34+a*28, 202+b*20, NeutralColor, 0);
                         E^.PutSprite(34+a*28, 402+b*20, NeutralColor, 0);
                       end;
  E^.PutSprite(146, 202, 6, 1);
  E^.PutSprite(146, 282, 6, 1);
  E^.PutSprite(146, 362, 6, 1);
  E^.PutSprite(34, 282, 6, 1);
  E^.PutSprite(258, 282, 6, 1);

  E^.PutSprite(146, 402, 6, 1);
  E^.PutSprite(146, 482, 6, 1);
  E^.PutSprite(146, 562, 6, 1);
  E^.PutSprite(34, 482, 6, 1);
  E^.PutSprite(258, 482, 6, 1);
  DrawSprites(1);
  ShowCursor;
end;

procedure MoveAround;
var Ch : Char;
begin
  repeat
    Ch := UpCase(ReadKey);
      case Ch of
        #72      : MoveCursor(1);
        #80      : MoveCursor(2);
        #75      : MoveCursor(3);
        #77      : MoveCursor(4);
      end;
  until (Ch = #32) or (Ch = #13);
end;

procedure MoveAround2;
var Ch : Char;
begin
  repeat
    Ch := UpCase(ReadKey);
      case Ch of
        'W'      : MoveCursor(1);
        'X'      : MoveCursor(2);
        'A'      : MoveCursor(3);
        'D'      : MoveCursor(4);
      end;
  until (Ch = 'S') or (Ch = #9);
end;

procedure CastSpell1;
var Ch        : Char;
    i, a, b,
    MaxKilled : Byte;
    Impr      : Boolean;
    St        : String;
    Art, x, y,
    x2, y2,
    Winner    : Word;
    BackGr    : BkgrPtr;
begin
  New(BackGr, InitBkgr(7, 20));
  Impr := False;
  i := 1;
  Str(Spell[1].Teleport, St);
  WriteMsg(SpellData[1]+'  '+St+'x');
  repeat
    Ch := UpCase(ReadKey);
    case Ch of
      #72 : if i > 1 then Dec(i)
              else i := 7;
      #80 : if i < 7 then Inc(i)
              else i := 1;
    end;
    case i of
      1 : Str(Spell[1].Teleport, St);
      2 : Str(Spell[1].Heal, St);
      3 : Str(Spell[1].ShiftTime, St);
      4 : Str(Spell[1].Exchange, St);
      5 : Str(Spell[1].Revive, St);
      6 : Str(Spell[1].Imprison, St);
    end;
    if i <> 7 then WriteMsg(SpellData[i]+'  '+St+'x')
      else WriteMsg(SpellData[i])
  until (Ch = #32) or (Ch = #13);
  case i of
    1 : if Spell[1].Teleport > 0 then begin
          PlChange := True;
          Dec(Spell[1].Teleport);
          WriteMsg('Select item to teleport!');
          repeat
            MoveAround;
          until ItemSet[(CursorY-2) div 20, (CursorX-34) div 28] <> 255;
          x := (CursorX-34) div 28; y := (CursorY-2) div 20;
          Art := ItemSet[y, x];
          Winner := Art;
          ItemSet[y, x] := 255;
          WriteMsg('Teleport item to...');
          repeat
            MoveAround;
          until ItemSet[(CursorY-2) div 20, (CursorX-34) div 28] >= 60;
          x2 := (CursorX-34) div 28; y2 := (CursorY-2) div 20;
          if ItemSet[y2, x2] <> 255 then InitFight(Art, x2, y2, Winner);
          ItemSet[y2, x2] := Winner;
          HideCursor;
          RedrawAll;
        end else begin
          WriteMsg('No more spells!');
          Delay(1500);
        end;
    2 : if Spell[1].Heal > 0 then begin
          for a := 0 to 8 do
            for b := 0 to 8 do
              if Imprisoned[a, b] then Impr := True;
          if Impr then begin
            PlChange := True;
            Dec(Spell[1].Heal);
            WriteMsg('Select an imprisoned item!');
            repeat
              MoveAround;
            until Imprisoned[(CursorY-2) div 20, (CursorX-34) div 28];
            Imprisoned[(CursorX-34) div 28, (CursorY-2) div 20] := False;
            WriteMsg('Item was healed!');
            Delay(2000);
          end else begin
                WriteMsg('Spell not possible!');
                Delay(2000);
              end;
        end else begin
          WriteMsg('No more spells!');
          Delay(1500);
        end;
    3 : if Spell[1].ShiftTime > 0 then begin
          PlChange := True;
          Dec(Spell[1].ShiftTime);
          WriteMsg('Time passes...');
          Delay(2000);
        end else begin
          WriteMsg('No more spells!');
          Delay(1500);
        end;
    4 : if Spell[1].Exchange > 0 then begin
          PlChange := True;
          Dec(Spell[1].Exchange);
          WriteMsg('Chose first item!');
          repeat
            MoveAround;
          until (ItemSet[(CursorY-2) div 20, (CursorX-34) div 28] <> 255) and
                (ItemSet[(CursorY-2) div 20, (CursorX-34) div 28] <> Wizard) and
                (ItemSet[(CursorY-2) div 20, (CursorX-34) div 28] <> DarkWizard);
          x := (CursorX-34) div 28; y := (CursorY-2) div 20;
          Art := ItemSet[y, x];
          WriteMsg('Chose second item!');
          repeat
            MoveAround;
          until (ItemSet[(CursorY-2) div 20, (CursorX-34) div 28] <> 255) and
                (ItemSet[(CursorY-2) div 20, (CursorX-34) div 28] <> Wizard) and
                (ItemSet[(CursorY-2) div 20, (CursorX-34) div 28] <> DarkWizard) and
                ((x <> (CursorX-34) div 28) or (y <> (CursorY-2) div 20));
          ItemSet[y, x] := ItemSet[(CursorY-2) div 20, (CursorX-34) div 28];
          ItemSet[(CursorY-2) div 20, (CursorX-34) div 28] := Art;
          HideCursor;
          RedrawAll;
        end else begin
          WriteMsg('No more spells!');
          Delay(1500);
        end;
    5 : if Spell[1].Revive > 0 then begin
          MaxKilled := 0;
          for i := 1 to 8 do if KilledItems[1, i] <> 255 then Inc(MaxKilled);
          if MaxKilled > 0 then begin
            Dec(Spell[1].Revive);
            PlChange := True;
            WriteMsg('Chose item to revive!');
            HideCursor;
            x := 2; y := 2;
            while KeyPressed do Ch := ReadKey;
            repeat
              BackGr^.SaveBkgr(x, y);
              Rect(x, y, 28, 20, 35);
              Rect(x+1, y+1, 26, 18, 37);
              Ch := ReadKey;
              WaitVRetrace;
              if Ch = #0 then Ch := ReadKey;
              BackGr^.RestoreBkgr(x, y);
              case Ch of
                #72 : if y > 002 then Dec(y, 20)
                        else y := 2+(MaxKilled-1)*20;
                #80 : if y < 2+(MaxKilled-1)*20 then Inc(y, 20)
                        else y := 002;
              end;
            until (Ch = #32) or (Ch = #13);
            Art := KilledItems[1, ((y-2) div 20)+1];
            KilledItems[1, ((y-2) div 20)+1] := 255;
            x2 := x; y2 := y;
            WriteMsg('Set item!');
            ShowCursor;
            repeat
              MoveAround;
              x := (CursorX-34) div 28; y := (CursorY-2) div 20;
            until (ItemSet[y, x] = 255) and
                  ((x <> 4) or (y <> 0)) and ((x <> 4) or (y <> 4)) and
                  ((x <> 4) or (y <> 8)) and ((x <> 0) or (y <> 4)) and
                  ((x <> 8) or (y <> 4));
            ItemSet[y, x] := Art;
            HideCursor;
            E^.PutSprite(x2, y2, White, 0);
            E^.PutSprite(x2, 200+y2, White, 0);
            E^.PutSprite(x2, 400+y2, White, 0);
            RedrawAll;
          end else begin
            WriteMsg('No killed items!');
            Delay(2000);
          end;
        end else begin
          WriteMsg('No more spells!');
          Delay(1500);
        end;
    6 : if Spell[1].Imprison > 0 then begin
          WriteMsg('Chose item to imprison!');
          repeat
            MoveAround;
            x := (CursorX-34) div 28; y := (CursorY-2) div 20;
          until (ItemSet[y, x] >= 60) and (ItemSet[y, x] <> 255);
          if not Imprisoned[x, y] then begin
            Imprisoned[x, y] := True;
            PlChange := True;
            Dec(Spell[1].Imprison);
            WriteMsg('Item is imprisoned!');
          end else begin
            WriteMsg('Already imprisoned!');
          end;
          Delay(2000);
        end else begin
          WriteMsg('No more spells!');
          Delay(1500);
        end;
  end;
  ClearMsg;
  Dispose(BackGr, DisposeBkgr);
end;

procedure CastSpell2;
var Ch        : Char;
    i, a, b,
    MaxKilled : Byte;
    Impr      : Boolean;
    St        : String;
    Art, x, y,
    x2, y2,
    Winner    : Word;
    BackGr    : BkgrPtr;
begin
  New(BackGr, InitBkgr(7, 20));
  Impr := False;
  i := 1;
  Str(Spell[2].Teleport, St);
  WriteMsg(SpellData[1]+'  '+St+'x');
  repeat
    Ch := UpCase(ReadKey);
    case Ch of
      'W' : if i > 1 then Dec(i)
              else i := 7;
      'X' : if i < 7 then Inc(i)
              else i := 1;
    end;
    case i of
      1 : Str(Spell[2].Teleport, St);
      2 : Str(Spell[2].Heal, St);
      3 : Str(Spell[2].ShiftTime, St);
      4 : Str(Spell[2].Exchange, St);
      5 : Str(Spell[2].Revive, St);
      6 : Str(Spell[2].Imprison, St);
    end;
    if i <> 7 then WriteMsg(SpellData[i]+'  '+St+'x')
      else WriteMsg(SpellData[i])
  until (Ch = 'S') or (Ch = #9);
  case i of
    1 : if Spell[2].Teleport > 0 then begin
          PlChange := True;
          Dec(Spell[2].Teleport);
          WriteMsg('Select item to teleport!');
          repeat
            MoveAround2;
          until ItemSet[(CursorY-2) div 20, (CursorX-34) div 28] <> 255;
          x := (CursorX-34) div 28; y := (CursorY-2) div 20;
          Art := ItemSet[y, x];
          Winner := Art;
          ItemSet[y, x] := 255;
          WriteMsg('Teleport item to...');
          repeat
            MoveAround2;
          until (ItemSet[(CursorY-2) div 20, (CursorX-34) div 28] <= 60) or
                (ItemSet[(CursorY-2) div 20, (CursorX-34) div 28] = 255);
          x2 := (CursorX-34) div 28; y2 := (CursorY-2) div 20;
          if ItemSet[y2, x2] <> 255 then InitFight(Art, x2, y2, Winner);
          ItemSet[y2, x2] := Winner;
          HideCursor;
          RedrawAll;
        end else begin
          WriteMsg('No more spells!');
          Delay(1500);
        end;
    2 : if Spell[2].Heal > 0 then begin
          for a := 0 to 8 do
            for b := 0 to 8 do
              if Imprisoned[a, b] then Impr := True;
          if Impr then begin
            PlChange := True;
            Dec(Spell[2].Heal);
            WriteMsg('Select an imprisoned item!');
            repeat
              MoveAround2;
            until Imprisoned[(CursorY-2) div 20, (CursorX-34) div 28];
            Imprisoned[(CursorX-34) div 28, (CursorY-2) div 20] := False;
            WriteMsg('Item was healed!');
            Delay(2000);
          end else begin
                WriteMsg('Spell not possible!');
                Delay(2000);
              end;
        end else begin
          WriteMsg('No more spells!');
          Delay(1500);
        end;
    3 : if Spell[2].ShiftTime > 0 then begin
          PlChange := True;
          Dec(Spell[2].ShiftTime);
          WriteMsg('Time passes...');
          Delay(2000);
        end else begin
          WriteMsg('No more spells!');
          Delay(1500);
        end;
    4 : if Spell[2].Exchange > 0 then begin
          PlChange := True;
          Dec(Spell[2].Exchange);
          WriteMsg('Chose first item!');
          repeat
            MoveAround2;
          until (ItemSet[(CursorY-2) div 20, (CursorX-34) div 28] <> 255) and
                (ItemSet[(CursorY-2) div 20, (CursorX-34) div 28] <> Wizard) and
                (ItemSet[(CursorY-2) div 20, (CursorX-34) div 28] <> DarkWizard);
          x := (CursorX-34) div 28; y := (CursorY-2) div 20;
          Art := ItemSet[y, x];
          WriteMsg('Chose second item!');
          repeat
            MoveAround2;
          until (ItemSet[(CursorY-2) div 20, (CursorX-34) div 28] <> 255) and
                (ItemSet[(CursorY-2) div 20, (CursorX-34) div 28] <> Wizard) and
                (ItemSet[(CursorY-2) div 20, (CursorX-34) div 28] <> DarkWizard) and
                ((x <> (CursorX-34) div 28) or (y <> (CursorY-2) div 20));
          ItemSet[y, x] := ItemSet[(CursorY-2) div 20, (CursorX-34) div 28];
          ItemSet[(CursorY-2) div 20, (CursorX-34) div 28] := Art;
          HideCursor;
          RedrawAll;
        end else begin
          WriteMsg('No more spells!');
          Delay(1500);
        end;
    5 : if Spell[2].Revive > 0 then begin
          MaxKilled := 0;
          for i := 1 to 8 do if KilledItems[2, i] <> 255 then Inc(MaxKilled);
          if MaxKilled > 0 then begin
            Dec(Spell[2].Revive);
            PlChange := True;
            WriteMsg('Chose item to revive!');
            HideCursor;
            x := 290; y := 2;
            while KeyPressed do Ch := ReadKey;
            repeat
              BackGr^.SaveBkgr(x, y);
              Rect(x, y, 28, 20, 35);
              Rect(x+1, y+1, 26, 18, 37);
              Ch := UpCase(ReadKey);
              WaitVRetrace;
              BackGr^.RestoreBkgr(x, y);
              case Ch of
                'W' : if y > 002 then Dec(y, 20)
                        else y := 2+(MaxKilled-1)*20;
                'X' : if y < 2+(MaxKilled-1)*20 then Inc(y, 20)
                        else y := 002;
              end;
            until (Ch = 'S') or (Ch = #9);
            Art := KilledItems[2, ((y-2) div 20)+1];
            KilledItems[2, ((y-2) div 20)+1] := 255;
            x2 := x; y2 := y;
            WriteMsg('Set item!');
            ShowCursor;
            repeat
              MoveAround2;
              x := (CursorX-34) div 28; y := (CursorY-2) div 20;
            until (ItemSet[y, x] = 255) and
                  ((x <> 4) or (y <> 0)) and ((x <> 4) or (y <> 4)) and
                  ((x <> 4) or (y <> 8)) and ((x <> 0) or (y <> 4)) and
                  ((x <> 8) or (y <> 4));
            ItemSet[y, x] := Art;
            HideCursor;
            E^.PutSprite(x2, y2, Black, 0);
            E^.PutSprite(x2, 200+y2, Black, 0);
            E^.PutSprite(x2, 400+y2, Black, 0);
            RedrawAll;
          end else begin
            WriteMsg('No killed items!');
            Delay(2000);
          end;
        end else begin
          WriteMsg('No more spells!');
          Delay(1500);
        end;
    6 : if Spell[2].Imprison > 0 then begin
          WriteMsg('Chose item to imprison!');
          repeat
            MoveAround2;
            x := (CursorX-34) div 28; y := (CursorY-2) div 20;
          until ItemSet[y, x] < 60;
          if not Imprisoned[x, y] then begin
            Imprisoned[x, y] := True;
            PlChange := True;
            Dec(Spell[2].Imprison);
            WriteMsg('Item is imprisoned!');
          end else begin
            WriteMsg('Already imprisoned!');
          end;
          Delay(2000);
        end else begin
          WriteMsg('No more spells!');
          Delay(1500);
        end;
  end;
  ClearMsg;
  Dispose(BackGr, DisposeBkgr);
end;

procedure StartGame;
var Ch             : Char;
    NeutDarken,
    Pl1Won, Pl2Won : Boolean;
    TurnCount      : Byte;
    i, j, x, y     : Word;
    GO             : array[0..129, 0..64] of Byte;
begin
  TurnCount := 0;
  Pl1Won := False; Pl2Won := False;
  if NeutralColor < 5 then NeutDarken := True
    else NeutDarken := False;
  Blend(False);
  RMove(0, 400, 320, 200, 0, 000);
  RMove(0, 400, 320, 200, 0, 200);
  DrawNeutralColor;
  SetScrPos(0, 0);
  Screen(0);
  SetVGAPal(ArchonPal, 0, 256);
  if LightFirst then SetOverscan(193)
    else SetOverscan(145);
  Blend(True);
  ShowCursor;
  repeat
    PlChange := False;
    repeat
      Ch := UpCase(ReadKey);
      if Pl1Turn and (Ch = #0) then begin
        Ch := ReadKey;
        case Ch of
          #68 : CastSpell1;
          #72 : MoveCursor(1);
          #80 : MoveCursor(2);
          #75 : MoveCursor(3);
          #77 : MoveCursor(4);
        end;
      end else if Pl1Turn then case Ch of
        #32, #13 : if MoveSprite then PlChange := True;
      end;
      if (not Pl1Turn) and (Ch = #0) then begin
        Ch := ReadKey;
        case Ch of
          #59 : CastSpell2;
        end
      end else if not Pl1Turn then case Ch of
        'W'     : MoveCursor(1);
        'X'     : MoveCursor(2);
        'A'     : MoveCursor(3);
        'D'     : MoveCursor(4);
        'S', #9 : if MoveSprite then PlChange := True;
      end;
    until PlChange or (Ch = #27);
    if (ItemSet[0, 4] <= 53) and (ItemSet[4, 4] <= 53) and
       (ItemSet[8, 4] <= 53) and (ItemSet[4, 0] <= 53) and
       (ItemSet[4, 8] <= 53) then Pl1Won := True;
    if (ItemSet[0, 4] >= 60) and (ItemSet[4, 4] >= 60) and
       (ItemSet[8, 4] >= 60) and (ItemSet[4, 0] >= 60) and
       (ItemSet[4, 8] >= 60) and
       (ItemSet[0, 4] < 255) and (ItemSet[4, 4] < 255) and
       (ItemSet[8, 4] < 255) and (ItemSet[4, 0] < 255) and
       (ItemSet[4, 8] < 255) then Pl2Won := True;
    Pl1Turn := not Pl1Turn;
    if Pl1Turn then begin
      SetOverscan(193);
      HideCursor;
      CursorX := 34; CursorY := 82;
      ShowCursor;
    end else begin
      SetOverscan(145);
      HideCursor;
      CursorX := 258; CursorY := 82;
      ShowCursor;
    end;
    Inc(TurnCount);
    if TurnCount = 2 then begin
      TurnCount := 0;
      if NeutDarken then Inc(NeutralColor)
        else Dec(NeutralColor);
      if NeutralColor = 5 then NeutDarken := False;
      if NeutralColor = 0 then NeutDarken := True;
      HideCursor;
      DrawNeutralColor;
      ShowCursor;
    end;
  until (Ch = #27) or Pl1Won or Pl2Won;
  GameOver^.PutSprite(105, 265, 0, 1);
  GameOver^.PutSprite(100, 295, 1, 1);
  for i := 0 to 129 do
    for j := 0 to 64 do GO[i, j] := GetPixel(100+i, 265+j);
  i := 0;
  repeat
    x := Random(129); y := Random(64);
    SetPixel(100+x, 65+y, GO[x, y]);
    Inc(i);
  until i = 20000;
  WaitVRetrace;
  GameOver^.PutSprite(105, 65, 0, 1);
  GameOver^.PutSprite(100, 95, 1, 1);
  ClearMsg;
  if Ch = #27 then WriteMsg('Game was aborted...');
  if Pl1Won then WriteMsg('The Light side wins!');
  if Pl2Won then WriteMsg('The Dark side wins!');
  while KeyPressed do Ch := ReadKey;
  repeat until KeyPressed;
  Blend(False);
  Done;
end;

begin
  GetIntVec($08, @OldInt8);
  SetIntVec($08, @NewInt8);
end.